#!/usr/bin/bash
#######################################################################
#Script Name: da_setup_deb.sh
#Version: 1.6
#Description: Wrapper for installing Da server
#Last Modify Date: 03102021
#Author:Brent Dacus
#Email:brent[at]thedacus[dot]net
#######################################################################
#                   		Banner                                        #
#######################################################################
export COLUMNS=100
dasetup_banner() {
	cat <<"eot"
                        ad88888ba
                       d8"     "8b              ,d
DdadPPYba,            Y8,                       88
HH      8b  ,dPPYba,  `Y8aaaaa,    ,adPPYba, MM88MMM  88       88 8b,dPPYba,
a8      44 88P'    "8a   `"""""8b, a8P_____88   88    88       88 88P'    "8a
8b      55 88       d8         `8b 8PP"""""""   88    88       88 88       d8
D8      aa 88      'b8 Y8a     a8P "8b,   ,aa   88,   "8a,   ,a88 88b,   ,a8"
TTTYbbd8"' `lLYbbdP''ag "Y88888P"   `"Ybbd8"'   "Y888  `"YbbdP'Y8 88`YbbdP"'
                                                                  88
                                                                  88
eot
	cat <<"eot"
			Author: Brent Dacus
eot
}
#######################################################################
#                       Variables                                     #
#######################################################################
cur_hostname="$(hostname)"
serverip="$(hostname -I | awk '{print $1}')"
serverip6="$(hostname -I | awk '{print $2}')"
servername="$(hostname -s)"
svrdomainname="$(hostname -d)"
hstdomainname=delainhosting.com
#os=$(uname)
#vn=$(uname -v | tr -dc '0-9.' | cut -d \. -f1)
cpu_cores=1
cpu_cores="$(cat /proc/cpuinfo | grep -F 'model name' | wc -l)"
logfile=/root/install.log
builddir=~/dasetupbuild/
dadir=/usr/local/directadmin
dascptdir=/usr/local/directadmin/scripts
datpldir=/usr/local/directadmin/data/templates/custom
datadmdir=/usr/local/directadmin/data/admin
cbblddir=/usr/local/directadmin/custombuild
daconfile=/usr/local/directadmin/conf/directadmin.conf
da=/usr/local/directadmin/directadmin
cb=/usr/local/directadmin/custombuild/build
aptargs="-y"
export DA_EMAIL="tech@delainhosting.com"
#######################################################################
#                      User Variables                                 #
#######################################################################
rootemail="tech@delainhosting.com"
kc_key="JOfCWN9tFkOYi1sl"
bshrc="/root/.bashrc"
#######################################################################
#                       Main Util Functions                           #
#######################################################################
mkdir -p ~/dasetupbuild/
trap '' 2 # ignore ctrl+c
##set PS3 prompt##
PS3="Number selection? "

linebreak() {
	printf '%*s\n' "${COLUMNS:-$(tput cols)}" '' | tr ' ' -
}
random_pass() {
	#No special characters yet, because they'd cause problems with regexes and PEAR::DB to split the DNS string correctly in roundcube config.inc.php
	rpc=${1:-$(perl -le 'print int rand(7) + 10')}
	tr -cd 'a-za-z0-9' </dev/urandom 2>/dev/null | head -c${rpc} # perl generates a random integer between 10 and 16
}
doreboot() {
	printf "Need to reboot? (y/n)? "
	read -r yn
	yn=${yn:-n}
	case $yn in
	[Yy]*)
		reboot
		;;
	[Nn]*) ;;
	esac
}
cycleswap() {
	printf "Cycling Swapfile.\n"
	swapoff -a
	sleep 10
	swapon -a
}
#######################################################################
#                       Add Admin User                                #
#######################################################################
addadminuser() {
	printf "Installing Standard packages.\n"
	printf "Please Wait.\n"
	apt-get install ${aptargs} perl5 wget curl nano htop >/dev/null
	bshrvals=("alias fbc='fail2ban-client status '" "export EDITOR='nano'")
	printf 'Adding values in to %s.\n Please wait.\n' "$bshrc"
	for bshrval in "${bshrvals[@]}"; do
		if grep -q "$bshrval" ${bshrc}; then
			printf '%s exists.\n Skipping.\n' "$bshrval"
		else
			printf '%s does not exist\n' "$bshrval"
			printf '#added by DH.\n' >>${bshrc}
			printf '%s\n' "$bshrval" >>${bshrc}
		fi
	done
	grep -q daconf /root/.bashrc
	# if not then create it
	if [ $? -ne 0 ]; then
		printf 'Bash shell profile not set up. adding...\n'
		cat <<"eol" >>/root/.bashrc
export EDITOR='nano'
export LS_OPTIONS='--color=auto'
eval "`dircolors`"
alias ls='ls $LS_OPTIONS'
alias ll='ls $LS_OPTIONS -l'
alias l='ls $LS_OPTIONS -lA'
alias rm='rm -i'
alias cp='cp -i'
alias mv='mv -i'
daconf='/usr/local/directadmin/conf/directadmin.conf'
cbconf='/usr/local/directadmin/custombuild/options.conf'
alias da='/usr/local/directadmin/directadmin'
alias dadir='pushd /usr/local/directadmin/'
alias dashdir='pushd /usr/local/directadmin/scripts'
alias cbdir='pushd /usr/local/directadmin/custombuild'
alias cb='/usr/local/directadmin/custombuild/build'
alias cbconfig='/usr/local/directadmin/custombuild/build used_configs'
alias cbopts='/usr/local/directadmin/custombuild/build options'
alias cbhelp='/usr/local/directadmin/custombuild/build opt_help full'
alias cbvers='/usr/local/directadmin/custombuild/build versions'
alias mysqladmin='mysqladmin --defaults-extra-file=/usr/local/directadmin/conf/my.cnf'
alias mysqldump='mysqldump --defaults-extra-file=/usr/local/directadmin/conf/my.cnf'
alias mysql='mysql --defaults-extra-file=/usr/local/directadmin/conf/my.cnf'
alias lh='history |grep ' 
alias search='find / -name '
alias doserver='curl -o da_setup_deb.sh -L https://files.delaintech.com/da_setup_deb.sh && bash da_setup_deb.sh'
eol
		printf "Print Profile File.\n"
		cat /root/.bashrc
		printf "Done.\n"
	else
		printf "Profile already setup.  See Below.\n"
		cat /root/.bashrc
	fi
}
#######################################################################
#                       Turn on IPV6                                  #
#######################################################################
setipv6() {
	printf "Do we need to turn on IPV6? REBOOT REQUIRED (y/n)? "
	read -r yn
	yn=${yn:-n}
	case $yn in
	[Yy]*)
		sed -i "/net.ipv6.conf.all.disable_ipv6.*/d" /etc/sysctl.conf
		sysctl -q -p
		echo 0 >/proc/sys/net/ipv6/conf/all/disable_ipv6
		${da} set ipv6 1
		doreboot
		systemctl restart directadmin
		;;
	[Nn]*) ;;
	esac
}
#######################################################################
#                        Set Timezone                                 #
#######################################################################
settimezone() {
	printf "What TimeZone are you in? [America/Chicago]: "
	read -r tmzone
	tmzone=${tmzone:-America/Chicago}
	grep -E "^$tmzone" /etc/timezone >/dev/null
	if [ $? -eq 0 ]; then
		printf "%s found\n" "$tmzone"
	else
		timedatectl set-timezone "$tmzone"
		printf "We set timezone as:\n"
		timedatectl
		printf "Done.\n"
	fi
}
#######################################################################
#                       Add Hosts file                                #
#######################################################################
creathostfile() {
	# does the Host already exist?
	grep -q ${hstdomainname} /etc/hosts
	# if not then create it
	if [ $? -ne 0 ]; then
		printf 'Hostfile not found. adding...\n'
		cat <<"eol" >>/etc/hosts
209.126.81.64           apollo.delainhosting.com apollo
209.145.52.110          athena.delainhosting.com athena
144.91.108.77           thor.delainhosting.com thor
116.202.102.0           saturn.delainhosting.com saturn
eol
		printf "Print Host File.\n"
		cat /etc/hosts
		printf "Done.\n"
	else
		printf "Hostfile exsits.\nSee Below.\n"
		cat /etc/hosts
	fi
}
#######################################################################
#                       Set Server Hostname                           #
#######################################################################
creathostname() {
	setipv6
	# does the Host already exist?
	unset new_hostname
	printf "Please enter a Hostname to add: "
	read -r new_hostname
	grep -q "$new_hostname" /etc/hosts >/dev/null
	if [ $? -ne 0 ]; then
		printf 'Hostname not found. adding...\n'
		printf "Changing hostname %s from to %s...\n" "$cur_hostname" "$new_hostname"
		hostnamectl set-hostname "$new_hostname"
		serverip="$(hostname -I | awk '{print $1}')"
		serverip6="$(hostname -I | awk '{print $2}')"
		servername="$(hostname -s)"
		echo "${serverip}           ${new_hostname} ${servername}" >>/etc/hosts
		echo "${serverip6}          ${new_hostname} ${servername}" >>/etc/hosts
		printf "Print Host File.\n"
		cat /etc/hosts
		printf "\nDone.\n"
	else
		printf "Hostname exsits.\nAll good.\n"
	fi
}
#######################################################################
#                       Remove Hostnames                              #
#######################################################################
removehosts() {
	printf "Here is the Host file.\n"
	cat /etc/hosts
	printf "\nWhat is the server name or ip to remove? "
	read -r removehosts
	sed -i.bkp '/'$removehosts'/d
' /etc/hosts
	printf "Print Host.\n"
	cat /etc/hosts
	printf "\nDone.\n"
	while true; do
		printf "Continue removing? (y/n)?"
		read -r yn
		yn=${yn:-n}
		case $yn in
		[Yy]*)
			removehosts
			break
			;;
		[Nn]*) break ;;
		esac
	done
}
#######################################################################
#                       Add External Hosts                            #
#######################################################################
addhosts() {
	# does the Host already exist?
	unset serverip add_hostname servername
	printf "Enter Hostname to add:[ IP FDQN Hostname ]: "
	read -r serverip add_hostname servername
	grep -q "$add_hostname" /etc/hosts
	# if not then create it
	if [ $? -ne 0 ]; then
		printf 'Hostname not found. adding...\n'
		printf "${serverip}			${add_hostname} ${servername}" | tee -a /etc/hosts
		printf "Print Host File.\n"
		cat /etc/hosts
		printf "Done.\n"
	else
		printf "Hostname exsits.\nAll good.\n"
	fi
	while true; do
		printf "Continue adding? (y/n)?"
		read -r yn
		yn=${yn:-n}
		case $yn in
		[Yy]*)
			creathostentry
			break
			;;
		[Nn]*) break ;;
		esac
	done
}
#######################################################################
#                       Add Swapfile                                  #
#######################################################################
creatswapfile() {
	printf 'Enter Swapfile size in GB: '
	read -r swapsize
	swapsize=${swapsize:-2}
	printf "You choose %s GB for swap.\n" "$swapsize"
	# does the swap file already exist?
	cp /etc/fstab /etc/fstab.bak
	# if not then create it
	grep -q "swap" /etc/fstab
	if [ $? -ne 0 ]; then
		printf 'Swap file not found.\nCreating Swap file.\n'
		fallocate -l ${swapsize}G /swapfile
		chmod 600 /swapfile
		mkswap /swapfile
		swapon /swapfile
		printf "/swapfile none swap defaults 0 0" | tee -a /etc/fstab
		printf "\nDone\nSwap should be active.\nIf not reboot.\n"
	else
		printf 'Swap file found.\nNo changes made.\n'
	fi
}
#######################################################################
#                       Update OS on Server                           #
#######################################################################
serverupdate() {
	printf "Debian Cleaning and Updating.\n"
	apt-get ${aptargs} autoremove
	apt-get autoclean
	apt-get ${aptargs} update
	apt-get ${aptargs} upgrade
	apt-get ${aptargs} dist-upgrade
	printf "Done.\n"
}
#######################################################################
#                       Install Standard Packages                     #
#######################################################################
installpreq() {
	#DA Standard packages
	apt-get install ${aptargs} curl wget gcc g++ make flex bison openssl libssl-dev perl perl-base \
		perl-modules-5.32 libperl-dev libperl4-corelibs-perl libwww-perl libaio1 libaio-dev \
		zlib1g zlib1g-dev libcap-dev cron bzip2 zip automake autoconf libtool cmake \
		pkg-config python3 libdb-dev libsasl2-dev libncurses5 libncurses5-dev libcurl4-openssl-dev \
		libsystemd-dev bind9 dnsutils quota patch logrotate rsyslog libc6-dev libexpat1-dev \
		libcrypt-openssl-rsa-perl libnuma-dev libnuma1 rspamd
	#DH Added Standard packages
	apt-get install ${aptargs} git apt-transport-https htop sysstat libcurl4-doc libidn11-dev \
		libkrb5-dev libldap2-dev librtmp-dev libssh2-1-dev screen inotify-tools ipset pigz tuned pcregrep rsync jq zip unzip
	linebreak
	printf "Installing required DirectAdmin Config!\n"
	mkdir -p ${cbblddir}
	cp ${cbblddir}/options.conf ${cbblddir}/options.conf.bak
	wget -rnH --cut-dirs=1 http://files.delaintech.com/deb/options.conf -P ${cbblddir} >>${logfile}
	wget -rnH --cut-dirs=1 http://files.delaintech.com/deb/php_extensions.conf -P ${cbblddir} >>${logfile}9
	linebreak
	printf "Here is what you have set for DA setup configs.\n"
	printf "Email set to %s.\n" "$DA_EMAIL"
	#printf "Nameservers set to %s and %s.\n" "$DA_NS1" "$DA_NS2"
	echo ${DA_EMAIL} >/root/.forward
	printf "Email forward set to %s.\n" "$DA_EMAIL"
	linebreak
	tuned-adm profile virtual-host
	printf "Done.\n"
}
#######################################################################
#                       Install Directadmin                           #
#######################################################################
installdirectadmin() {
	if [ ! -f "$daconfile" ]; then
		printf "No problem, let's get DirectAdmin installed first...\nThis could take a minute...\nor two... or thirty...\nPlease wait ...\n"
		cd /root || return
		export DA_EMAIL="tech@delainhosting.com" \
			export DA_NS1="dns3.${svrdomainname}" \
			export DA_NS2="dns2.${svrdomainname}" \
			bash <(curl -Ss https://www.directadmin.com/setup.sh) auto
	else
		printf "Directadmin already installed.\n"
	fi
	${cb} set_fastest_quiet
}
#######################################################################
#                       Harden Server                                 #
#######################################################################
hardenserver() {
	printf "Do we need to Secure sshd? (y/n)? "
	read -r yn
	yn=${yn:-n}
	case $yn in
	[Yy]*)
		cursshport="$(grep -m1 -E "Port .*" /etc/ssh/sshd_config)"
		printf "Enter SSH port to change to: "
		read -r sshport
		sshport=${sshport:-14}
		printf "Set to Port: %s\n" "$sshport"
		printf "Securing the server, please wait...\n"
		sed -i -e "s/$cursshport/Port ${sshport}/g" /etc/ssh/sshd_config
		sed -i -e 's/.*UseDNS .*/UseDNS no/g' /etc/ssh/sshd_config
		sed -i -e 's/#AddressFamily any/AddressFamily inet/g' /etc/ssh/sshd_config
		sed -i -e 's/#LoginGraceTime 2m/LoginGraceTime 2m/g' /etc/ssh/sshd_config
		sed -i -e 's/#MaxAuthTries 6/MaxAuthTries 5/g' /etc/ssh/sshd_config
		sed -i -e 's/#MaxStartups 10:30:100/MaxStartups 10:30:100/g' /etc/ssh/sshd_config
		sed -i -e 's/.*PermitRootLogin yes/PermitRootLogin prohibit-password/g' /etc/ssh/sshd_config
		sed -i -e 's/.*PasswordAuthentication .*/PasswordAuthentication no/g' /etc/ssh/sshd_config
		sed -i -e 's/#ClientAliveInterval .*/ClientAliveInterval 120/g' /etc/ssh/sshd_config
		sed -i -e 's/#ClientAliveCountMax .*/ClientAliveCountMax 15/g' /etc/ssh/sshd_config
		sed -i -e 's/.*UseBlacklist no/UseBlacklist yes/g' /etc/ssh/sshd_config
		# remove or disable services
		systemctl restart sshd
		;;
	[Nn]*) ;;
	esac
	if [[ $(systemctl is-active rpcbind) = active ]]; then
		printf "rpcbind found.\nRemoving.\n"
		systemctl stop rpcbind
		systemctl disable rpcbind
		printf "Removal complete.\n"
	else
		printf "RPCbind not found.\nNo changes made.\n"
	fi
	if [[ $(systemctl is-active firewalld) = active ]]; then
		printf "Firewalld found.\nRemoving.\n"
		systemctl unmask --now firewalld
		systemctl disable firewalld
		apt-get ${aptargs} remove firewalld
		printf "Removal complete.\n"
	else
		printf "Firewalld not found.\nNo changes made.\n"
	fi
	printf "Do we need to intall Maldetect? (y/n)? "
	read -r yn
	yn=${yn:-n}
	case $yn in
	[Yy]*)
		installmaldetect
		;;
	[Nn]*) ;;
	esac
	printf "Do we need to intall Fail2Ban? (y/n)? "
	read -r yn
	yn=${yn:-n}
	case $yn in
	[Yy]*)
		installf2b
		;;
	[Nn]*) ;;
	esac
	printf "Do we need to secure PHP? (y/n)? "
	read -r yn
	yn=${yn:-n}
	case $yn in
	[Yy]*)
		cd ${cbblddir} || exit
		mkdir -p custom
		touch custom/php_disable_functions
		echo "show_source, system, shell_exec, passthru, popen" >custom/php_disable_functions
		${cb} secure_php
		;;
	[Nn]*) ;;
	esac
}
#######################################################################
#                       Install Firewall                              #
#######################################################################
installfirewall() {
	touch /etc/whitelist_ips
	echo "99.34.232.208" >/etc/whitelist_ips
	echo "127.0.0.1" >>/etc/whitelist_ips
	echo "$serverip" >>/etc/whitelist_ips
	touch /etc/blocked_ips
	${da} set ip_blacklist /etc/blocked_ips
	${da} set ip_whitelist /etc/whitelist_ips
	printf "Securing the server, please wait...\n"

	if [ -f /usr/local/csf/bin/csftest.pl ]; then
		printf "csf is installed skipping."
	else
		cd $builddir
		rm -fv csf.tgz
		wget https://download.configserver.com/csf.tgz
		tar -xzf csf.tgz
		cd csf
		./install.sh
	fi
	# check perl modules
	perl /usr/local/csf/bin/csftest.pl

	cat >/etc/csf/csf.allow <<eol
99.34.232.208 # csf SSH installation/upgrade IP address
tcp|out|d=8081|s=${serverip}  #$cur_hostname
tcp|in|d=8081|s=144.91.108.77 #Thor 
tcp|in|d=8081|s=209.145.52.11 #Athena
tcp|in|d=2222|s=209.126.81.64 #Apollo
tcp|in|d=2222|s=116.202.102.0 #Saturn
eol
	cat >/etc/csf/csf.blocklists <<eol
# Spamhaus Don't Route Or Peer List (DROP)
# Details: http://www.spamhaus.org/drop/
SPAMDROP|86400|0|http://www.spamhaus.org/drop/drop.txt

# Spamhaus Extended DROP List (EDROP)
# Details: http://www.spamhaus.org/drop/
SPAMEDROP|86400|0|http://www.spamhaus.org/drop/edrop.txt

# Details: https://docs.abuseipdb.com/#blacklist-endpoint
ABUSEIPDB|86400|10000|https://api.abuseipdb.com/api/v2/blacklist?&key=c13866e213ef23a0fbf2c57cd6474d63477dda0767a2a088249de1151f269157c48ab70a4dc0c531&plaintext&limit=20000
eol
	csf -ra
}
#######################################################################
#                       Install Fail2Ban                              #
#######################################################################
installf2b() {
	if [[ $(systemctl is-active fail2ban) = active ]]; then
		printf "Fail2Ban already Installed.\nUpdating configuration.\n"
		sed -i -e "s|dbpurgeage = .*|dbpurgeage = 1296000|g" /etc/fail2ban/fail2ban.conf
		wget -rnH --cut-dirs=2 http://files.delaintech.com/deb/f2b/fail2ban -P /etc/logrotate.d/ >>${logfile}
		wget -rnH --cut-dirs=2 http://files.delaintech.com/deb/f2b/jail.local -P /etc/fail2ban/ >>${logfile}
		wget -rnH --cut-dirs=2 http://files.delaintech.com/deb/f2b/paths-overrides.local -P /etc/fail2ban/ >>${logfile}
		wget -rnH --cut-dirs=2 http://files.delaintech.com/deb/f2b/abuseipdb.conf -P /etc/fail2ban/action.d/ >>${logfile}
		wget -rnH --cut-dirs=2 http://files.delaintech.com/deb/f2b/csf-ip-deny.conf -P /etc/fail2ban/action.d/ >>${logfile}
		wget -rnH --cut-dirs=2 http://files.delaintech.com/deb/f2b/wordpress.conf -P /etc/fail2ban/filter.d/ >>${logfile}
		wget -rnH --cut-dirs=2 http://files.delaintech.com/deb/f2b/wordpress-xmlrpc.conf -P /etc/fail2ban/filter.d/ >>${logfile}
		sed -i -e "s|comment=Fail2ban Reports Abuse.|comment=Fail2ban at $servername Reports Abuse.|g" /etc/fail2ban/action.d/abuseipdb.conf
		sed -i -e "s|mode = .*|mode = aggressive|g" /etc/fail2ban/filter.d/exim.conf
		systemctl restart fail2ban
		systemctl status fail2ban
		${da} set bruteforce 0
		${da} set brute_force_log_scanner 0
		${da} set brute_force_scan_apache_logs 0
		systemctl restart directadmin
	else
		printf "Fail2Ban Installing.\nAlso turning off BFM in DA.\n"
		apt-get install ${aptargs} fail2ban geoip-bin geoip-database python3-pyinotify
		systemctl enable fail2ban
		systemctl start fail2ban
		sed -i -e "s|dbpurgeage = .*|dbpurgeage = 1296000|g" /etc/fail2ban/fail2ban.conf
		wget -rnH --cut-dirs=2 http://files.delaintech.com/deb/f2b/jail.local -P /etc/fail2ban/ >>${logfile}
		wget -rnH --cut-dirs=2 http://files.delaintech.com/deb/f2b/paths-overrides.local -P /etc/fail2ban/ >>${logfile}
		wget -rnH --cut-dirs=2 http://files.delaintech.com/deb/f2b/abuseipdb.conf -P /etc/fail2ban/action.d/ >>${logfile}
		wget -rnH --cut-dirs=2 http://files.delaintech.com/deb/f2b/csf-ip-deny.conf -P /etc/fail2ban/action.d/ >>${logfile}
		wget -rnH --cut-dirs=2 http://files.delaintech.com/deb/f2b/wordpress.conf -P /etc/fail2ban/filter.d/ >>${logfile}
		wget -rnH --cut-dirs=2 http://files.delaintech.com/deb/f2b/wordpress-xmlrpc.conf -P /etc/fail2ban/filter.d/ >>${logfile}
		sed -i -e "s|comment=Fail2ban Reports Abuse.|comment=Fail2ban at $servername Reports Abuse.|g" /etc/fail2ban/action.d/abuseipdb.conf
		sed -i -e "s|mode = .*|mode = aggressive|g" /etc/fail2ban/filter.d/exim.conf
		systemctl restart fail2ban
		${da} set bruteforce 0
		${da} set brute_force_log_scanner 0
		${da} set brute_force_scan_apache_logs 0
		systemctl restart directadmin
	fi
	bshrvals=("alias fbc='fail2ban-client status '")
	printf 'Adding values in to %s.\n Please wait.\n' "$bshrc"
	for bshrval in "${bshrvals[@]}"; do
		if grep -q "$bshrval" ${bshrc}; then
			printf '%s exists.\n Skipping.\n' "$bshrval"
		else
			printf '%s does not exist\n' "$bshrval"
			printf '#added by DH.\n' >>${bshrc}
			printf '%s\n' "$bshrval" >>${bshrc}
		fi
	done
}
#######################################################################
#                       Install LetsEncrypt                           #
#######################################################################
installletsencrypt() {
	resolvedip=$(dig "${cur_hostname}" | awk '/^;; ANSWER SECTION:$/ { getline ; print $5 }')
	if [ -n "${resolvedip}" ]; then
		printf 'Creating add sni. adding...'
		${da} set enable_ssl_sni 1
		${da} set mail_sni 1
		printf "Installing Letsencrypt.\n"
		${da} set letsencrypt 1
		echo "action=directadmin&value=restart" >>/usr/local/directadmin/data/task.queue
		/usr/local/directadmin/dataskq d2000
		${cb} set "redirect_host ${cur_hostname}"
		${cb} set redirect_host_https yes
		${cb} rewrite_confs
		${cb} update
		${cb} letsencrypt
		#set ssl on server
		printf "Installing SSL to server.\n"
		cd ${dadir}/scripts || exit
		./letsencrypt.sh request_single "${cur_hostname}" 4096
		${da} set ssl 1
		${da} set force_hostname "${cur_hostname}"
		${da} set ssl_redirect_host "${cur_hostname}"
		${da} set letsencrypt_list www:webmail:mail:ftp
		${da} set letsencrypt_list_selected www:webmail:mail:ftp
		${da} set letsencrypt_renewal_notice_to_admins 0
		${da} set admin_ssl_install_to_missing 1
		${da} set admin_ssl_replace_all_expired_invalid 1
		systemctl restart directadmin
		printf "Fingers crossed..if your server resolves to the name it should have worked.\n"
	else
		printf "Nope..if your server does not resolve. Check DNS.. \n"
	fi
}
#######################################################################
#                       Install Mysqltuner                            #
#######################################################################
installmysqltuner() {
	cd /usr/bin || exit
	wget http://mysqltuner.pl/ -O mysqltuner
	wget https://raw.githubusercontent.com/major/MySQLTuner-perl/master/basic_passwords.txt -O basic_passwords.txt
	wget https://raw.githubusercontent.com/major/MySQLTuner-perl/master/vulnerabilities.csv -O vulnerabilities.csv
	chmod +x mysqltuner
}
#######################################################################
#                       Install Maldetect                             #
#######################################################################
installmaldetect() {
	cd $builddir || return
	wget https://www.rfxn.com/downloads/maldetect-current.tar.gz
	tar -xzf maldetect-*.tar.gz
	rm -rf maldetect-*.tar.gz
	cd maldetect* || return
	sh install.sh
	wget https://files.delaintech.com/conf.maldet -O conf.maldet
	\cp -f conf.maldet /usr/local/maldetect/
	maldet -u
}
#######################################################################
#                       Install Mail Queue                            #
#######################################################################
installmailqueue() {
	cd $builddir || exit
	wget http://download.configserver.com/cmq.tgz
	tar -xzf cmq.tgz
	cd cmq || exit
	sh install.sh
}
#######################################################################
#                       Install Installatron                          #
#######################################################################
installinstallatron() {
	cd $builddir || exit
	wget https://data.installatron.com/installatron-plugin.sh
	chmod +x installatron-plugin.sh
	./installatron-plugin.sh -f
}
#######################################################################
#                       Install KernelCare                            #
#######################################################################
installkernelcare() {
	#Is Linux Kernel Compatible?
	printf "Linux Kernel Compatible?\n"
	comp="$(curl -s -L https://kernelcare.com/checker | python3)"
	#What install KernelCare
	if [ "$comp" = "COMPATIBLE" ]; then
		printf "Yes..System Kernel Compatible.\nContinue on....\n"
		curl -s -L https://kernelcare.com/installer | bash
		printf "Kernel Care installed.\n"
		printf 'Enter KernelCare license key: '
		read -r kckey
		kc_key=${kckey:-JOfCWN9tFkOYi1sl}
		/usr/bin/kcarectl --register $kc_key
		/usr/bin/kcarectl --update
	else
		printf "System runs on unsupported kernel. Not installed...\n"
	fi
}
#######################################################################
#                       Main Configure Section                        #
#######################################################################
#######################################################################
#                       Configure Directadmin                         #
#######################################################################
confdirectadmin() {
	mkdir ${datpldir}
	wget -rnH http://files.delaintech.com/nginx_server.conf -P ${datpldir} >>${logfile}
	wget -rnH http://files.delaintech.com/nginx_server_secure.conf -P ${datpldir} >>${logfile}
	wget -rnH http://files.delaintech.com/nginx_server_sub.conf.CUSTOM.4.post -P ${datpldir} >>${logfile}
	wget -rnH http://files.delaintech.com/nginx_server_secure.conf.CUSTOM.4.post -P ${datpldir} >>${logfile}
	wget -rnH http://files.delaintech.com/nginx_server_secure_sub.conf.CUSTOM.4.post -P ${datpldir} >>${logfile}
	wget -rnH http://files.delaintech.com/virtual_host2_secure.conf.CUSTOM.4.post -P ${datpldir} >>${logfile}
	wget -rnH http://files.delaintech.com/virtual_host2.conf.CUSTOM.4.post -P ${datpldir} >>${logfile}
	wget -rnH http://files.delaintech.com/virtual_host2_secure_sub.conf.CUSTOM.4.post -P ${datpldir} >>${logfile}
	wget -rnH http://files.delaintech.com/virtual_host2_sub.conf.CUSTOM.4.post -P ${datpldir} >>${logfile}
	wget -rnH http://files.delaintech.com/php-fpm.conf -P ${datpldir} >>${logfile}
	mkdir -p custom/opcache
	wget -rnH http://files.delaintech.com/opcache.ini -P ${datpldir}/opcache >>${logfile}
	${cb} opcache
	${cb} nginx_apache
	${cb} mod_htscanner2
	systemctl enable directadmin
	cp /usr/local/directadmin/scripts/setup.txt /usr/local/directadmin/scripts/setupdh.txt
	wget -rnH http://files.delaintech.com/dns_a.conf -P ${datpldir} >>${logfile}
	wget -rnH http://files.delaintech.com/dns_aaaa.conf -P ${datpldir} >>${logfile}
	wget -rnH http://files.delaintech.com/dns_ns.conf -P ${datpldir} >>${logfile}
	wget -rnH http://files.delaintech.com/dns_txt.conf -P ${datpldir} >>${logfile}
	wget -rnH http://files.delaintech.com/user_create_post.sh -P ${dadir}/scripts/custom >>${logfile}
	chmod 755 ${dadir}/scripts/custom/user_create_post.sh
	chown -R diradmin:diradmin ${datpldir}
	cp "$daconfile" "$daconfile".bak
	${cb} secure_php
	${da} set zstd 1
	${da} set backup_gzip 1
	${da} set zip 1
	${da} set pigz ${cpu_cores}
	${da} set enforce_difficult_passwords 1
	${da} set difficult_password_length_min 8
	${da} set check_subdomain_owner 1
	${da} set admin_helper clients.delainhosting.com
	${da} set cluster 1
	${da} set enable_threads 1
	${da} set cpu_in_system_info 1
	${da} set delete_messages_days 1
	${da} set delete_tickets_days 1
	${da} set msg_sys "Delain Hosting"
	${da} set update_channel current
	${da} set jail 0
	systemctl restart directadmin
	${cb} rewrite_confs
}
#######################################################################
#                       Configure Email                               #
#######################################################################
confmail() {
	printf "Rebuild all of email? (y/n)? "
	read -r yn
	yn=${yn:-n}
	case $yn in
	[Yy]*)
		printf 'Creating add sni. adding...\n'
		${da} set enable_ssl_sni 1
		${da} set mail_sni 1
		${da} set dkim 1
		${da} set spam_inbox_prefix 0
		${da} set purge_spam_days 30
		${cb} set webapps_inbox_prefix no
		echo "EASY_NO_REVERSE_IP==50" >/etc/exim.easy_spam_fighter/variables.conf.custom
		echo "EASY_SPF_FAIL==50" >>/etc/exim.easy_spam_fighter/variables.conf.custom
		echo "EASY_DKIM_FAIL==10" >>/etc/exim.easy_spam_fighter/variables.conf.custom
		echo "ssl=required" >/etc/dovecot/conf.d/force_ssl.conf
		#mail quota warning
		cd /etc/dovecot/conf.d || return
		wget -O 91-quota-warning.conf http://files1.directadmin.com/services/all/91-quota-warning.conf
		wget -O /usr/local/bin/quota-warning.sh http://files1.directadmin.com/services/all/quota-warning.sh
		chmod 755 /usr/local/bin/quota-warning.sh

		#end mail quota warning
		#remove rbl checking
		cd /etc/virtual || return
		rm -f use_rbl_domains
		touch use_rbl_domains
		#Custom MX templates
		cd ${dadir}/data/templates/mx || exit
		mkdir -p custom
		rm -f /usr/local/directadmin/data/templates/mx/custom/*
		wget -rnd --no-parent -A '*.txt' http://files.delaintech.com/custom/ -P /usr/local/directadmin/data/templates/mx/custom/
		chown diradmin:diradmin '*.txt'
		#Webmail client choice
		${da} set webmail_link roundcube
		${cb} set roundcube yes
		${cb} set squirrelmail no
		printf 'Creating add sni. adding...\n'
		${da} set enable_ssl_sni 1
		${da} set mail_sni 1
		${cb} clean
		${cb} update
		${cb} dovecot
		${cb} exim
		${cb} pigeonhole
		${cb} exim_conf
		${cb} dovecot_conf
		${cb} blockcracking
		${cb} roundcube
		;;
	[Nn]*) ;;
	esac
	printf "Set Root mail? (y/n)? "
	read -r yn
	yn=${yn:-n}
	case $yn in
	[Yy]*)
		printf "Where should root email go: "
		read -r rootemail
		rootemail=${rootemail:-tech@delainhosting.com}
		grep -E "$rootemail" /etc/aliases >/dev/null
		if [ $? -eq 0 ]; then
			printf "%s found already set in Aliases file.\n" "$rootemail"
		else
			sed -i -e "s|.*[[:blank:]]*root:[[:blank:]].*|root: ${rootemail}|g" /etc/aliases
			newaliases
			printf "Email set to: "
			grep -E "$rootemail" /etc/aliases
		fi
		echo "$cur_hostname" >/etc/mailname
		;;
	[Nn]*) ;;
	esac
	printf "Disable POP mail? (y/n)? "
	read -r yn
	yn=${yn:-y}
	case $yn in
	[Yy]*)
		touch /etc/exim.variables.conf.custom
		grep -qxF 'hostlist relay_hosts=' /etc/exim.variables.conf.custom || echo 'hostlist relay_hosts=' >>/etc/exim.variables.conf.custom
		;;
	[Nn]*) ;;
	esac
	printf "Setup standard mail settings? (y/n)? "
	read -r yn
	yn=${yn:-n}
	case $yn in
	[Yy]*)
		printf 'Creating add sni. adding...\n'
		${da} set mail_sni 1
		${da} set dkim 1
		${da} set spam_inbox_prefix 0
		${da} set purge_spam_days 30
		${da} set mx_templates 1
		${cb} set webapps_inbox_prefix no
		echo "EASY_NO_REVERSE_IP==50" >/etc/exim.easy_spam_fighter/variables.conf.custom
		echo "EASY_SPF_FAIL==50" >>/etc/exim.easy_spam_fighter/variables.conf.custom
		echo "EASY_DKIM_FAIL==10" >>/etc/exim.easy_spam_fighter/variables.conf.custom
		echo "ssl=required" >/etc/dovecot/conf.d/force_ssl.conf
		#mail quota warning
		cd /etc/dovecot/conf.d || return
		wget -O 91-quota-warning.conf http://files1.directadmin.com/services/all/91-quota-warning.conf
		wget -O /usr/local/bin/quota-warning.sh http://files1.directadmin.com/services/all/quota-warning.sh
		chmod 755 /usr/local/bin/quota-warning.sh
		#end mail quota warning
		#remove rbl checking
		cd /etc/virtual || return
		rm -f use_rbl_domains
		touch use_rbl_domains
		#Custom MX templates
		cd ${dadir}/data/templates/mx || exit
		mkdir -p custom
		rm -f /usr/local/directadmin/data/templates/mx/custom/*
		wget -rnd --no-parent -A '*.txt' http://files.delaintech.com/custom/ -P /usr/local/directadmin/data/templates/mx/custom/
		chown diradmin:diradmin '*.txt'
		#Webmail client choice
		${da} set webmail_link roundcube
		${cb} set roundcube yes
		${cb} set squirrelmail no
		${cb} roundcube
		;;
	esac
	mkdir -p ${dadir}/custombuild/custom/roundcube/
	cp /var/www/html/roundcube/config/config.inc.php ${cbblddir}/custom/roundcube/config.inc.php
	sed -i -e 's|"'localhost'"|tls://%n|g' ${cbblddir}/custom/roundcube/config.inc.php
	sed -i -e 's/Roundcube Webmail/Delain Hosting Webmail/g' ${cbblddir}/custom/roundcube/config.inc.php

	cat >/etc/dovecot/conf.d/90-special-folders.conf <<"eol"
namespace inbox {
  type = private
  separator = .
  subscriptions = yes
  inbox = yes
}
namespace inbox {
  mailbox Drafts {
    auto = subscribe
    special_use = \Drafts
  }
  mailbox Junk {
   auto = subscribe
   special_use = \Junk
}
mailbox Archive {
   auto = subscribe
   special_use = \Archive
}
mailbox Trash {
   auto = subscribe
   special_use = \Trash
}
mailbox Sent {
   auto = subscribe
   special_use = \Sent
}
}
eol
	#End Webmail client choice
	systemctl restart directadmin
	${cb} exim_conf
	${cb} dovecot_conf
	${cb} rewrite_confs
	read -p "Install Mail Queue? (yn)" yn
	case $yn in
	[Yy]*)
		installmailqueue
		;;
	[Nn]*) ;;
	esac
}
#######################################################################
#                       Configure DNS                                 #
#######################################################################
confnamed() {
	#sed -i -e "s|.*listen-on-v6[[:blank:]]*.{ any; };||g" /etc/bind/named.conf.options
	sed -i -e "s|.*allow-transfer[[:blank:]]*.{ none; };||g" /etc/bind/named.conf.options
	cd ${dascptdir} || exit
	./dnssec.sh install
	grep -q "recursion no;" /etc/bind/named.conf.options
	if [ $? -ne 0 ]; then
		cp /etc/bind/named.conf.options /etc/bind/named.conf.options.bak
		sed -i.bkp '/dnssec-validation yes;/a\
	listen-on { '$serverip'; };\
	listen-on-v6 { '$serverip6'; };\
	dnssec-enable yes;\
	dnssec-lookaside auto;\
	bindkeys-file "/etc/bind/named.iscdlv.key";\
	recursion no;\
	notify yes;\
	allow-transfer  { 209.145.52.110; 144.91.108.77; };\
	also-notify     { 209.145.52.110; 144.91.108.77; };\
	version "Not Found";\
' /etc/bind/named.conf.options
	fi
	${da} set ns1 "dns3.${svrdomainname}"
	${da} set ns2 "dns2.${svrdomainname}"
	${da} set dns_ttl 1
	${da} set default_ttl 43200

	apt-get remove resolvconf
	echo 'make_resolv_conf() { :; }' >/etc/dhcp/dhclient-enter-hooks.d/leave_my_resolv_conf_alone
	chmod 755 /etc/dhcp/dhclient-enter-hooks.d/leave_my_resolv_conf_alone
	printf "Setup Resolver configuration.\n"
	grep -q "9.9.9.9" /etc/resolv.conf
	if [ $? -ne 0 ]; then
		cat >/etc/resolv.conf <<eol
search domain $svrdomainname
nameserver 127.0.0.1
nameserver 9.9.9.9
nameserver 149.112.112.112
eol
	fi
	printf "Restarting Services.\n"
	systemctl enable named
	systemctl restart named
	systemctl restart directadmin
	printf "Done.\n"
}
#######################################################################
#                       Configure Proftpd with SFTP                   #
#######################################################################
confproftpd() {
	printf "Setting up Proftpd.\n"
	cd ${dadir} || return
	${da} set unified_ftp_password_file 1
	echo "action=convert&value=unifiedftp" >>data/task.queue
	./dataskq d1
	${cb} update
	${cb} set ftpd proftpd
	cd ${cbblddir} || return
	mkdir -p custom/{proftpd,conf}
	wget -rnH --cut-dirs=2 http://files.delaintech.com/deb/proftpd/configure.proftpd -P ${cbblddir}/custom/proftpd >>${logfile}
	wget -rnH --cut-dirs=2 http://files.delaintech.com/deb/proftpd/proftpd.conf -P ${cbblddir}/custom/proftpd/conf >>${logfile}
	chmod 755 custom/proftpd/configure.proftpd
	wget -rnH --cut-dirs=2 http://files.delaintech.com/deb/proftpd/proftpd.sftp.conf -P /etc >>${logfile}
	sed -i -e "s|<VirtualHost 0.0.0.0>|<VirtualHost ${serverip}>|g" /etc/proftpd.sftp.conf
	${cb} proftpd
	printf "Setting up Proftp.\n"
	systemctl enable proftpd
	systemctl restart proftpd
	printf "Done.\n"
}
#######################################################################
#                       Configure MariaDB                             #
#######################################################################
confmysql() {
	if ! grep -q 'local-infile=0' /etc/my.cnf; then
		echo 'local-infile=0' >>/etc/my.cnf
		echo 'bind-address = 127.0.0.1' >>/etc/my.cnf
		systemctl restart mysqld
	fi
	printf "Do we need to intall Mysqltuner? (y/n)? "
	read -r yn
	yn=${yn:-n}
	case $yn in
	[Yy]*)
		installmysqltuner
		;;
	[Nn]*) ;;
	esac
	${da} set one_click_pma_login 1
}
#######################################################################
#                       Check Server Satus                            #
#######################################################################
serverstatus() {
	printf "Checking Directadmin."
	systemctl status directadmin --no-pager
	printf "done"
}
#######################################################################
#                       Rebuild Directadmin                           #
#######################################################################
buildalld() {
	printf 'Running Build All D...Go sleep or get Coffee!'
	${cb} set_fastest_quiet
	${cb} clean
	${cb} update
	${cb} all
	${cb} rewrite_confs
	printf "Build ALL done.\n"
}
#######################################################################
#                       Backup Server                                 #
#######################################################################
backupall() {
	wget -rnH --cut-dirs=1 http://files.delaintech.com/deb/all_backups_post.sh -P ${dadir}/scripts/custom
	chmod 755 ${dascptdir}/custom/all_backups_post.sh
	wget -rnH http://files.delaintech.com/backup.conf -P ${datadmdir}
	wget -rnH http://files.delaintech.com/backup_crons.list -P ${datadmdir}
	printf "Install Rclone for file backup.(yn)"
	read -r yn
	yn=${yn:-n}
	case $yn in
	[Yy]*)
		printf 'Installing Rclone for file backup.\n'
		apt-get install ${aptargs} rclone
		mkdir -p /var/log/rclone
		touch /var/log/rclone/aws.log
		printf "Setup Rclone.\n"
		rclone config
		;;
	[Nn]*) ;;
	esac
	printf 'Running Server file backup.\n'
	if [ -d /sys_backup ] && [ -d /admin_backups ]; then
		chmod 755 /admin_backups
		chmod 755 /sys_backup
		chown admin:admin /admin_backups
		chown admin:admin /sys_backup
		cd /
		tar -cvpzf /sys_backup/backup.tar.gz \
			--exclude=/sys_backup/backup.tar.gz \
			--exclude=/home/admin/admin_backup \
			--exclude=/admin_backups \
			--exclude=/root/.c* \
			/home/admin/hb* \
			/usr/local/directadmin/conf \
			/usr/local/directadmin/scripts/custom \
			/usr/local/directadmin/custombuild/custom \
			/usr/local/directadmin/data/templates/custom \
			/usr/local/directadmin/data/templates/mx \
			/usr/local/directadmin/data/admin/packages/ \
			/usr/local/directadmin/data/admin/packages.list \
			/usr/local/directadmin/data/users/admin/packages/ \
			/usr/local/directadmin/data/users/admin/packages.list \
			/usr/local/directadmin/data/admin/backup* \
			/sys_backup/mysql_backups \
			/etc \
			/root
		printf "Server Backup done.\n"
	else
		>/home/admin/hb_templates_c/hostbillqueue.log
		rm -Rf /home/admin/admin_backup
		mkdir /admin_backups
		mkdir /sys_backup
		chmod 755 /admin_backups
		chmod 755 /sys_backup
		chown admin:admin /admin_backups
		cd /
		tar -cvpzf /sys_backup/backup.tar.gz \
			--exclude=/sys_backup/backup.tar.gz \
			--exclude=/home/admin/admin_backup \
			--exclude=/admin_backups \
			--exclude=/root/.c* \
			--exclude=/sys_backup/backup.tar.gz \
			--exclude=/admin_backups \
			--exclude=/root/.c* \
			/home/admin/hb* \
			/usr/local/directadmin/conf \
			/usr/local/directadmin/scripts/custom \
			/usr/local/directadmin/custombuild/custom \
			/usr/local/directadmin/data/templates/custom \
			/usr/local/directadmin/data/templates/mx \
			/usr/local/directadmin/data/admin/packages/ \
			/usr/local/directadmin/data/admin/packages.list \
			/usr/local/directadmin/data/users/admin/packages/ \
			/usr/local/directadmin/data/users/admin/packages.list \
			/usr/local/directadmin/data/admin/backup* \
			/sys_backup/mysql_backups \
			/etc \
			/root
		printf "Server Backup done.\n"
	fi
	/usr/local/directadmin/scripts/custom/all_backups_post.sh
}
#######################################################################
#                       Main Menu of Script                           #
#######################################################################
clear
dasetup_banner
linebreak

#Check that user is root.
if [ $EUID = "0" ]; then
	printf "We are root. Continue on....\n"
else
	printf "This script must be run as root\n"
	exit 1
fi
#What Distro are you on?
printf "Distro are you on??\n" 2>&1
OS_VER=$(sed 's/\..*//' /etc/debian_version)
if [ "$OS_VER" = 9 ]; then
	printf "This appears to be Debian version %s which is Good!!\n" "$OS_VER"
elif [ "$OS_VER" = 10 ]; then
	printf "This appears to be Debian version %s which is Better!!\n" "$OS_VER"
elif [ "$OS_VER" = 11 ]; then
	printf "This appears to be Debian version %s which is Great!!\n" "$OS_VER"
else
	[ "$OS_VER" != 11 ]
	printf "System runs on unsupported Linux. Exiting...\n"
	exit
fi
#Menus Starts here
linebreak
printf "${YELLOW}"
cat <<eot
	Hostname: $cur_hostname Server IP: $serverip
eot
linebreak
printf "${YELLOW}"
cat <<"eot"
			MAIN SERVER SETUP MENU
eot
linebreak
while true; do
	options=("Add Admin User" "Timezone" "Host file" "Hostname" "Remove hosts" "Add hosts" "Swap file" "Update server" "Install Required Packages" "Harden Server" "Install Directadmin" "Install Firewall" "Install LetsEncrypt" "Install KernelCare" "Install MalDetect" "Install MySQLTuner" "Install Installatron" "Configure Directadmin" "Configure Named" "Configure Mail" "Configure Proftpd" "Configure MySql" "Check Server Status" "Build All" "Backup Server Files" "Quit")
	printf "Choose an option: \n"
	select opt in "${options[@]}"; do
		case $REPLY in
		1)
			addadminuser
			break
			;;
		2)
			settimezone
			break
			;;
		3)
			creathostfile
			break
			;;
		4)
			creathostname
			break
			;;
		5)
			removehosts
			break
			;;
		6)
			addhosts
			break
			;;
		7)
			creatswapfile
			break
			;;
		8)
			serverupdate
			break
			;;
		9)
			installpreq
			break
			;;
		10)
			hardenserver
			break
			;;
		11)
			installdirectadmin
			break
			;;
		12)
			installfirewall
			break
			;;
		13)
			installletsencrypt
			break
			;;
		14)
			installkernelcare
			break
			;;
		15)
			installmaldetect
			break
			;;
		16)
			installmysqltuner
			break
			;;
		17)
			installinstallatron
			break
			;;
		18)
			confdirectadmin
			break
			;;
		19)
			confnamed
			break
			;;
		20)
			confmail
			break
			;;
		21)
			confproftpd
			break
			;;
		22)
			confmysql
			break
			;;
		23)
			serverstatus
			break
			;;
		24)
			buildalld
			break
			;;
		25)
			backupall
			break
			;;
		26) break 2 ;;
		*) echo "Invalid option. Try again." >&2 ;;
		esac
	done
done

if [ -d "$builddir" ]; then
	rm -rf $builddir
fi
mkdir $builddir
if [ "$run" ]; then
	${run}
	exit
fi

printf "Cleaning up build files, please wait...\n"
cd ~ || exit
rm -rf $builddir
