#!/bin/bash
#######################################################################
#Script Name: da_setup_arch.sh
#Version: 1.6
#Description: Wrapper for installing Da server
#Last Modify Date: 03102021
#Author:Brent Dacus
#Email:brent[at]thedacus[dot]net
#######################################################################
#                   		Banner                                        #
#######################################################################
export COLUMNS=100
dsksetup_banner() {
	cat <<"eot"
                        ad88888ba
                       d8"     "8b              ,d
DdadPPYba,            Y8,                       88
HH      8b  ,dPPYba,  `Y8aaaaa,    ,adPPYba, MM88MMM  88       88 8b,dPPYba,
a8      44 88P'    "8a   `"""""8b, a8P_____88   88    88       88 88P'    "8a
8b      55 88       d8         `8b 8PP"""""""   88    88       88 88       d8
D8      aa 88      'b8 Y8a     a8P "8b,   ,aa   88,   "8a,   ,a88 88b,   ,a8"
TTTYbbd8"' `lLYbbdP''ag "Y88888P"   `"Ybbd8"'   "Y888  `"YbbdP'Y8 88`YbbdP"'
                                                                  88
                                                                  88
eot
	cat <<"eot"
			Author: Brent Dacus
eot
}
#######################################################################
#                       Variables                                     #
#######################################################################
cur_hostname="$(hostname)"
serverip="$(ifconfig | grep -E 'inet.[0-9]' | grep -v '127.0.0.1' | awk 'NR==1{ print $2}')"
servername="$(hostname -s)"
svrdomainname="$(hostname -d)"
admuser=bdacus01
os=$(uname)
vn=$(freebsd-version -u | tr -s '0-9.' | cut -d \. -f1)
pvn=$(freebsd-version -u | tr -s '0-9.' | cut -d - -f3)
prvn=$(freebsd-version -u | tr -s '0-9.' | cut -d - -f1)
logfile=/root/install.log
builddir=~/dsktpsetup/
usrdir=/usr/local/
usetcdir=/usr/local/etc/
rcconf=/etc/rc.conf
ldrconf=/boot/loader.conf

#######################################################################
#                       Main                                          #
#######################################################################
mkdir -p ~/dsktpsetup/
trap '' 2 # ignore ctrl+c
##set PS3 prompt##
PS3="Number selection? "

linebreak() {
	printf '%*s\n' "${COLUMNS:-$(tput cols)}" '' | tr ' ' -
}
random_pass() {
	rpc=${1:-$(perl -le 'print int rand(7) + 10')}
	tr -cd 'a-za-z0-9' </dev/urandom 2>/dev/null | head -c"${rpc}" # perl generates a random integer between 10 and 16
}
doreboot() {
	printf "Need to reboot? (y/n)? "
	read -r yn
	yn=${yn:-n}
	case $yn in
	[Yy]*)
		reboot
		;;
	[Nn]*) ;;
	esac
}
# ------------------------------------------------------------------------------#
# Install Functions 															#
# ------------------------------------------------------------------------------#
addadminuser() {
	printf "Installing Standard packages.\n"
	printf "Please Wait.\n"
	pacman -Sy doas nano htop remmina wget vscode  >/dev/null
	printf "Add an admin user? Enter username [bdacus01]:"
	read -r admuser
	admuser=${admuser:-bdacus01}
	grep -E "^$admuser\b" /etc/passwd >/dev/null
	if [ $? -eq 0 ]; then
		printf "User found: %s\n" "$admuser"
	else
		echo "$(random_pass)" | pw user add "$admuser" -m -s /bin/tcsh -G wheel -h 0
		echo "Created admin user" "$admuser" "with" "$(random_pass)" "."
		echo "$(random_pass)" >/home/"$admuser"/pass.txt
	fi
	printf "Done.\n"
	grep -q "#User Config" /home/"$admuser"/.cshrc
	# if not then create it
	if [ $? -ne 0 ]; then
		printf 'C shell profile not setup for %s. adding...\n' "$admuser"

		sed -i -e 's/.*EDITOR.*/setenv  EDITOR  nano/g' /home/"$admuser"/.cshrc
		cat <<"eol" >>/home/"$admuser"/.cshrc
#User Config
alias rm rm -i
alias cp cp -i
alias mv mv -i
alias df df -achT
alias usrdir cd /usr/local/
alias uetc cd /usr/local/etc/
alias etc cd /etc/
set rcf = /etc/rc.conf
set ldrf = /boot/loader.conf
alias la ls -abFG
alias lc ls -bCG
alias ll ls -abhlG
alias lr ls -bRG
alias lh 'history |grep ' 
alias search 'find / -name '
alias doansible cd /home/bdacus01/ansible/
eol
		printf "Print Profile File.\n"
		cat /home/"$admuser"/.cshrc
		printf "Done."
	else
		printf "Profile already setup for %s.\n" "$admuser"
	fi
	printf 'Shell profile not set up. adding Editor...\n'
	printf "EDITOR=nano\nexport EDITOR\n" | tee -a /home/"$admuser"/.profile

	grep -q "#Root Config" /root/.cshrc
	# if not then create it
	if [ $? -ne 0 ]; then
		printf 'C shell profile not setup for Root. adding...\n'

		sed -i -e 's/.*EDITOR.*/setenv  EDITOR  nano/g' /root/.cshrc
		cat <<"eol" >>/root/.cshrc
#Root Config
alias rm rm -i
alias cp cp -i
alias mv mv -i
alias df df -achT
alias usrdir cd /usr/local/
alias uetc cd /usr/local/etc/
alias etc cd /etc/
set rcf = /etc/rc.conf
set ldrf = /boot/loader.conf
alias la ls -abFG
alias lc ls -bCG
alias ll ls -abhlG
alias lr ls -bRG
alias lh 'history |grep ' 
alias search 'find / -name '
alias top top -P
alias dodesk 'curl -o dsktp_setup_bsd.sh -L https://files.delaintech.com/dsktp_setup_bsd.sh && bash dsktp_setup_bsd.sh'
eol
		printf "Print Profile File.\n"
		cat /root/.cshrc
		printf "Done.\n"
	else
		printf "Profile already setup for Root.\n"
	fi
	printf 'Shell profile not set up. adding Editor...\n'
	printf "EDITOR=nano\nexport EDITOR\n" | tee -a /root/.profile
}
settimezone() {
	printf "What TimeZone are you in? [America/Chicago]: "
	read -r tmzone
	tmzone=${tmzone:-America/Chicago}
	grep -E "^$tmzone" /var/db/zoneinfo >/dev/null
	if [ $? -eq 0 ]; then
		printf "%s found\n" "$tmzone"
	else
		tzsetup "$tmzone"
	fi
	printf "We set timezone as: "
	cat /var/db/zoneinfo
	printf "Done.\n"
}
creathostfile() {
	# does the Host already exist?
	grep -q "$svrdomainname" /etc/hosts
	# if not then create it
	if [ $? -ne 0 ]; then
		printf 'Hostfile not found. adding...\n'
		#-----------------------------------------------------------------------------#
		# Set your hostfile info inside the eol																				#
		#----------------------------------------------------------------------------
		cat <<"eol" >>/etc/hosts
209.126.81.64           apollo.delainhosting.com apollo
209.145.52.110          athena.delainhosting.com athena
207.244.97.182          thor.delainhosting.com thor
116.202.102.0           saturn.delainhosting.com saturn
eol
		printf "Print Host File.\n"
		cat /etc/hosts
		printf "Done.\n"
	else
		printf "Hostfile exsits.\nSee Below.\n"
		cat /etc/hosts
	fi
}
#-----------------------------------------------------------------------------#
# Set hostname for Server fdqn.server.net																			#
#-----------------------------------------------------------------------------#
creathostname() {
	# does the Host alreadry exist?
	unset new_hostname
	printf "Please enter a Hostname to add: "
	read -r new_hostname
	grep -q "$new_hostname" "$rcconf" >/dev/null
	if [ $? -ne 0 ]; then
		printf 'Hostname not found. adding...\n'
		printf "Changing hostname %s from to %s...\n" "$cur_hostname" "$new_hostname"
		sysrc hostname="$new_hostname"
		serverip="$(ifconfig | grep -E 'inet.[0-9]' | grep -v '127.0.0.1' | awk 'NR==1{ print $2}')"
		servername="$(new_hostname | cut -d "." -f1)"
		echo "${servername}"
		echo "${serverip}           ${new_hostname} ${servername}" >>/etc/hosts
		printf "Print Host File.\n"
		cat /etc/hosts
		printf "Done.\n"
	else
		printf "Hostname exsits.\nAll good.\n"
	fi
}
#-----------------------------------------------------------------------------#
# create extra host enties for other servers fdqn.server.net									#
#-----------------------------------------------------------------------------#
creathostentry() {
	# does the Host already exist?
	unset serverip add_hostname servername
	printf "Enter Hostname to add:[ IP FDQN Hostname ]: "
	read -r serverip add_hostname servername
	grep -q "$add_hostname" /etc/hosts
	# if not then create it
	if [ $? -ne 0 ]; then
		printf 'Hostname not found. adding...\n'
		echo "${serverip}           ${add_hostname} ${servername}" >>/etc/hosts

		printf "Print Host File.\n"
		cat /etc/hosts
		printf "Done.\n"
	else
		printf "Hostname exsits.\nAll good.\n"
	fi
	while true; do
		printf "Continue adding? (y/n)?"
		read -r yn
		yn=${yn:-n}
		case $yn in
		[Yy]*)
			creathostentry
			break
			;;
		[Nn]*) break ;;
		esac
	done
}
removehosts() {
	printf "Here is the Host file.\n"
	cat /etc/hosts
	printf "What is the server name or ip to remove?"
	read -r removehosts
	sed -i.bkp '/'$removehosts'/d
' /etc/hosts
	printf "Print Host.\n"
	cat /etc/hosts
	printf "Done.\n"
	while true; do
		printf "Continue removing? (y/n)?"
		read -r yn
		yn=${yn:-n}
		case $yn in
		[Yy]*)
			removehosts
			break
			;;
		[Nn]*) break ;;
		esac
	done
}
#-----------------------------------------------------------------------------#
# Create Swap file 								        																		#
#-----------------------------------------------------------------------------#
creatswapfile() {
	printf 'Enter Swapfile size in GB: '
	read -r swapsize
	swapsize=${swapsize:-2}
	printf "You choose %s GB for swap.\n" "$swapsize"
	# does the swap file already exist?
	cp /etc/fstab /etc/fstab.bak
	# if not then create it
	grep -q "swap" /etc/fstab
	if [ $? -ne 0 ]; then
		printf 'Swap file not found.\nCreating Swap file.\n'
		truncate -s "$swapsize"G /swapfile
		chmod 0600 /swapfile
		swapon -aq
		printf "md99 none swap sw,file=/swapfile,late 0 0" | tee -a /etc/fstab
		swapon -aqL
		swapinfo -g
		printf "Done Swap should be active.\nIf not reboot.\n"
	else
		printf 'Swap file found.\nNo changes made.\n'
	fi
}
serverupdate() {
	printf "Arch Linux Cleaning and Updating.\n"
	pacman autoremove -y >/dev/null
	pacman clean -a -y >/dev/null
	printf "Done.\n"
	printf "Updating Arch Linux.\nHold Please.\n"
	pacman -Syu
	doreboot
	printf "Done.\n"
}
#-----------------------------------------------------------------------------#
# Start configuring and hardening here   										                  #
#-----------------------------------------------------------------------------#
hardenserver() {
	printf "Do we need to Secure sshd? (y/n)? "
	read -r yn
	yn=${yn:-n}
	case $yn in
	[Yy]*)
		cursshport="$(grep -m1 -E "Port .*" /etc/ssh/sshd_config)"
		printf "Enter SSH port to change to:"
		read -r sshport
		sshport=${sshport:-14}
		printf "Set to Port: %s\n" "$sshport"
		printf "Securing the server, please wait...\n"
		sed -i -e "s/$cursshport/Port ${sshport}/g" /etc/ssh/sshd_config
		sed -i -e 's/.*UseDNS .*/UseDNS no/g' /etc/ssh/sshd_config
		sed -i -e 's/#AddressFamily any/AddressFamily inet/g' /etc/ssh/sshd_config
		sed -i -e 's/#LoginGraceTime 2m/LoginGraceTime 2m/g' /etc/ssh/sshd_config
		sed -i -e 's/#MaxAuthTries 6/MaxAuthTries 5/g' /etc/ssh/sshd_config
		sed -i -e 's/#MaxStartups 10:30:100/MaxStartups 10:30:100/g' /etc/ssh/sshd_config
		sed -i -e 's/.*PermitRootLogin yes/PermitRootLogin yes/g' /etc/ssh/sshd_config
		sed -i -e 's/.*PasswordAuthentication .*/PasswordAuthentication yes/g' /etc/ssh/sshd_config
		sed -i -e 's/.*PubkeyAuthentication .*/PubkeyAuthentication yes/g' /etc/ssh/sshd_config
		sed -i -e 's/#ClientAliveInterval .*/ClientAliveInterval 120/g' /etc/ssh/sshd_config
		sed -i -e 's/#ClientAliveCountMax .*/ClientAliveCountMax 15/g' /etc/ssh/sshd_config
		sed -i -e 's/.*UseBlacklist no/UseBlacklist yes/g' /etc/ssh/sshd_config
		# remove or disable services
		sysrc rpcbind_enable="NO"
		service rpcbind onestop
		service rpcbind onedisable
		service sshd restart
		;;
	[Nn]*) ;;
	esac
}

installpreq() {
	printf 'Setting up DOAS file.\n'
	grep -q "#DH doas file setup" /usr/local/etc/doas.conf
	if [ $? -ne 0 ]; then
		cat >/etc/doas.conf <<eol
#DH doas file setup
permit nopass :wheel
permit nopass $admuser as root
eol
		printf 'Done.\n'
	fi
	printf 'Setting admin groups.\n'
	pw groupmod wheel -m "$admuser"
	pw groupmod operator -m "$admuser"
	pw groupmod network -m "$admuser"

	ldrvals=('autoboot_delay="3"' 'boot_mute="YES"' 'kern.vty=vt' 'kern.vt.fb.default_mode="1280x720"' 'kern.ipc.shmseg=1024' 'kern.ipc.shmmni=1024' 'kern.maxproc=100000' 'hw.ata.atapi_dma="1"' 'aio_load="YES"' 'ahci_load="YES"' 'atapicam_load="YES"' 'tmpfs_load="YES"' 'coretemp_load="YES"' 'cpu_microcode_load="YES"' 'cpu_microcode_name="/boot/firmware/intel-ucode.bin"')
	printf 'Adding values in to %s.\n Please wait.\n' "$ldrconf"
	for ldrval in "${ldrvals[@]}"; do
		if grep -q "$ldrval" ${ldrconf}; then
			printf '%s exists.\n Skipping.\n' "$ldrval"
		else
			printf '%s does not exist\n' "$ldrval"
			printf '#added by DH.\n' >>${ldrconf}
			printf '%s\n' "$ldrval" >>${ldrconf}
		fi
	done

	printf 'Creating boot env.\n Please wait.\n'
	if beadm list | grep -q factory"$prvn"; then
		printf 'Exists. Skipping.\n'
	else
		printf 'Does not exist\n'
		beadm create factory"$prvn"
		printf 'Added. factory%s.\n' "$prvn"
	fi
	if beadm list | grep -q "desktop"$prvn" NR"; then
		printf 'Desktop BE is already Active.\n Skipping.\n'
	else
		printf 'Activating BE.\n'
		beadm activate factory"$prvn"
		printf 'Added. factory%s.\nYou might need to reboot.\n' "$prvn"
		doreboot
	fi
}

installdesktop() {
	printf 'Creating boot env.\n Please wait.\n'
	if beadm list | grep -q desktop"$prvn"; then
		printf 'Exists. Skipping.\n'
	else
		printf 'Does not exist\n'
		beadm create desktop"$prvn"
		printf 'Added. desktop%s.\n' "$prvn"
	fi
	if beadm list | grep -q "desktop"$prvn" NR"; then
		printf 'Desktop BE is already Active.\n Skipping.\n'
	else
		printf 'Activating BE.\n'
		beadm activate desktop"$prvn"
		printf 'Activated desktop%s.\nYou might need to reboot.\n' "$prvn"
		doreboot
	fi
	printf "Installing Desktop packages.\n"
	printf "Please Wait.\n"
	pacman -Sy xorg kde5 plasma5-plasma sddm drm-kmod plasma5-sddm-kcm cups screen rsync filezilla vscode remmina cdrtools dvd+rw-tools \
		fusefs-ntfs
	printf "Done.\n"
	printf "Install Ansible? (yn)"
	read -r yn
	yn=${yn:-n}
	case $yn in
	[Yy]*)
		printf 'Installing Ansible.\n'
		pacman -Sy py38-ansible virtualbox-ose virtualbox-ose-additions vagrant
		ldrvals=('vboxdrv_enable="YES"')
		printf 'Adding values in to %s.\n Please wait.\n' "$ldrconf"
		for ldrval in "${ldrvals[@]}"; do
			if grep -q "$ldrval" ${ldrconf}; then
				printf '%s exists.\n Skipping.\n' "$ldrval"
			else
				printf '%s does not exist\n' "$ldrval"
				printf '#added by DH.\n' >>${ldrconf}
				printf '%s\n' "$ldrval" >>${ldrconf}
				pw groupmod vboxusers -m "$admuser"
				pw groupmod operator -m "$admuser"
			fi
		done
		sysrc vboxnet_enable="YES"
		sysrc vboxguest_enable="YES"
		sysrc vboxservice_enable="YES"
		mkdir -p /usr/local/etc/ansible
		cp /usr/local/etc/ansible/hosts /usr/local/etc/ansible/hosts.bak
		cp /usr/local/etc/ansible/ansible.cfg /usr/local/etc/ansible/ansible.cfg.bak
		cp /usr/local/share/examples/py38-ansible-core/hosts /usr/local/etc/ansible
		cp /usr/local/share/examples/py38-ansible-core/ansible.cfg /usr/local/etc/ansible
		mkdir -p /home/"${admuser}"/ansible/{bsd/dns,deb/{da,vrtm},adhoc/{bsd,deb}}
		chown -R "${admuser}":"${admuser}" /home/"${admuser}"/ansible
		;;
	[Nn]*) ;;
	esac
}

confdesktop() {
	sysrc dbus_enable="YES" && service dbus start
	sysrc sddm_enable="YES" && service sddm start
	sysrc kld_list="amdgpu coretemp fusefs"

	pw groupmod video -m "$admuser"
}

confresolver() {
	printf "Setup Resolver configuration.\n"
	grep -q "192.168.59.1" /etc/resolv.conf
	if [ $? -ne 0 ]; then
		cat >/etc/resolv.conf <<eol
search domain $svrdomainname
nameserver 127.0.0.1
nameserver 192.168.59.1
eol
	fi
}
serverstatus() {
	printf "Checking Directadmin.\n"
	service dbus status
	service sddm status
	service sshd status
	printf "Done.\n"
}
backupall() {
	printf "Install Rclone for file backup.(yn)"
	read -r yn
	yn=${yn:-n}
	case $yn in
	[Yy]*)
		printf 'Installing Rclone for file backup.\n'
		pacman -Sy rclone pidof
		mkdir -p /var/log/rclone
		touch /var/log/rclone/aws.log
		printf "Setup Rclone.\n"
		rclone config
		;;
	[Nn]*) ;;
	esac
	printf 'Running Server file backup.\n'
	if [ -d /sys_backup ] && [ -d /admin_backups ]; then
		chmod 755 /admin_backups
		chmod 755 /sys_backup
		chown admin:admin /admin_backups
		chown admin:admin /sys_backup
		cd /
		tar -czvf /sys_backup/backup.tgz --exclude sys_backup/backup.tar.gz --exclude home/admin/admin_backup --exclude admin_backups --exclude root/*c* usr/local/directadmin/conf usr/local/directadmin/scripts/custom usr/local/directadmin/custombuild/custom usr/local/directadmin/data/templates/custom usr/local/directadmin/data/templates/mx usr/local/directadmin/data/admin/packages/ usr/local/directadmin/data/admin/packages.list usr/local/directadmin/data/users/admin/packages/ usr/local/directadmin/data/users/admin/packages.list /usr/local/directadmin/data/admin/backup* sys_backup/mysql_backups usr/local/etc etc root
	else
		mkdir -p /admin_backups
		mkdir -p /sys_backup
		chmod 755 /admin_backups
		chmod 755 /sys_backup
		chown admin:admin /admin_backups
		chown admin:admin /sys_backup
		cd /
		tar -czvf /sys_backup/backup.tgz --exclude sys_backup/backup.tar.gz --exclude home/admin/admin_backup --exclude admin_backups --exclude root/*c* usr/local/directadmin/conf usr/local/directadmin/scripts/custom usr/local/directadmin/custombuild/custom usr/local/directadmin/data/templates/custom usr/local/directadmin/data/templates/mx usr/local/directadmin/data/admin/packages/ usr/local/directadmin/data/admin/packages.list usr/local/directadmin/data/users/admin/packages/ usr/local/directadmin/data/users/admin/packages.list /usr/local/directadmin/data/admin/backup* sys_backup/mysql_backups usr/local/etc etc root

		printf "Server Backup done.\n"
	fi
}

#-------------------------------------------------------------------------------#
# Main Desktop Functions 										 											    		#
# ------------------------------------------------------------------------------#
clear
dsksetup_banner
linebreak

#Check that user is root.
if [ "$(id -u)" = "0" ]; then
	printf "We are root. Continuing on....\n"
else
	printf "This script must be run as root. Exiting.\n"
	exit 1
fi
#What Distro are you on?
printf "This installer is for Arch Linux 12+.\n" 2>&1
if [ "$os" = "Arch Linux" ] && [ "$vn" -gt 12 ]; then
	printf "System runs on %s version %s patch level %s. Great! Continuing on....\n" "$os" "$prvn" "$pvn"
else
	printf "System runs on unsupported os. Exiting...\n"
	exit
fi
#Menus Starts here
linebreak
cat <<eot
	Hostname: $cur_hostname Server IP: $serverip
eot
linebreak
cat <<"eot"
			MAIN DA SERVER SETUP MENU
eot
linebreak
while true; do
	options=("Add Admin User" "Timezone" "Host file" "Hostname" "Add hosts" "Remove hosts" "Swap file" "Update server" "Install Factory settings" "Harden Server" "Install Firewall" "Install Desktop" "Configure Desktop" "Configure Resolver" "Check Server Status" "Backup Server Files" "Quit")
	echo "Choose an option: "
	select opt in "${options[@]}"; do
		case $REPLY in
		1)
			addadminuser
			break
			;;
		2)
			settimezone
			break
			;;
		3)
			creathostfile
			break
			;;
		4)
			creathostname
			break
			;;
		5)
			creathostentry
			break
			;;
		6)
			removehosts
			break
			;;
		7)
			creatswapfile
			break
			;;
		8)
			serverupdate
			break
			;;
		9)
			installpreq
			break
			;;
		10)
			hardenserver
			break
			;;
		11)
			installfirewall
			break
			;;
		12)
			installdesktop
			break
			;;
		13)
			confdesktop
			break
			;;
		14)
			confresolver
			break
			;;
		15)
			serverstatus
			break
			;;
		16)
			backupall
			break
			;;
		17) break 2 ;;
		*) echo "Invalid option. Try again." >&2 ;;
		esac
	done
done

if [ -d "$builddir" ]; then
	rm -rf $builddir
fi
mkdir $builddir
if [ "$run" ]; then
	${run}
	exit
fi

printf "Cleaning up build files, please wait...\n"
cd ~ || return
rm -rf $builddir
