#!/usr/bin/bash
#######################################################################
#Script Name: vm_setup_deb.sh
#Version: 1.6
#Description: Wrapper for installing Da server
#AbuseIPDB
#Last Modify Date: 03102021
#Author:Brent Dacus
#Email:brent[at]thedacus[dot]net
#######################################################################
#                   		Banner                                        #
#######################################################################
export COLUMNS=100
vmsetup_banner() {
	cat <<"eot"
                         ad88888ba
                        d8"     "8b              ,d
Dd       ,VM           Y8,                       88
 HH      8b ,dPj ,dYa. `Y8aaaaa,    ,adPPYba,  MM88MMM 88       88 8b,dPPYba,
  a8    44 8P'  V   8b   `"""""8b,  a8P_____88   88    88       88 88P'    "8a
   8b  55  88       d8         ` 8b 8PP"""""""   88    88       88 88       d8
    'Da'   88       b8 Y8a      a8P "8b,   ,aa   88,   "8a,   ,a88 88b,   ,a8"
     V     TT       YM   "Y88888P"   `"Ybbd8"'   "Y888  `"YbbdP'Y8 88`YbbdP"'
                                                                   88
                                                                   88
eot
	cat <<"eot"
			Author: Brent Dacus
eot
}
#######################################################################
#                       Variables                                     #
#######################################################################
cur_hostname="$(hostname)"
serverip="$(hostname -I | awk '{print $1}')"
serverip6="$(hostname -I | awk '{print $2}')"
servername="$(hostname -s)"
svrdomainname="$(hostname -d)"
hstdomainname=delainhosting.com
#os=$(uname)
#vn=$(uname -v | tr -dc '0-9.' | cut -d \. -f1)

cpu_cores=1
cpu_cores="$(cat /proc/cpuinfo | grep -F 'model name' | wc -l)"
logfile=/root/install.log
aptargs="-y"
csf_conf="/etc/csf/csf.conf"
vmsetupbuild="/root/vmsetupbuild"
vm="/usr/sbin/virtualmin"
#######################################################################
#                      User Variables                                 #
#######################################################################
rootemail="tech@delainhosting.com"
kc_key="JOfCWN9tFkOYi1sl"
#######################################################################
#                       Main                                          #
#######################################################################
mkdir -p ~/vmsetupbuild/
trap '' 2 # ignore ctrl+c
##set PS3 prompt##
PS3="Number selection? "

linebreak() {
	printf '%*s\n' "${COLUMNS:-$(tput cols)}" '' | tr ' ' -
}
random_pass() {
	#No special characters yet, because they'd cause problems with regexes and PEAR::DB to split the DNS string correctly in roundcube config.inc.php
	rpc=${1:-$(perl -le 'print int rand(7) + 10')}
	tr -cd 'a-za-z0-9' </dev/urandom 2>/dev/null | head -c${rpc} # perl generates a random integer between 10 and 16
}
doreboot() {
	printf "Need to reboot? (y/n)? "
	read -r yn
	yn=${yn:-n}
	case $yn in
	[Yy]*)
		reboot
		;;
	[Nn]*) ;;
	esac
}

# ------------------------------------------------------------------------------#
# Install Functions 															#
# ------------------------------------------------------------------------------#
addadminuser() {
	printf "Installing Standard packages.\n"
	printf "Please Wait.\n"
	apt-get install ${aptargs} wget curl nano htop

	grep -q vmconf /root/.bashrc
	# if not then create it
	if [ $? -ne 0 ]; then
		printf 'Bash shell profile not set up. adding...\n'
		#-----------------------------------------------------------------------------#
		# Set your hostfile info inside the eol																				#
		#----------------------------------------------------------------------------
		cat <<"eol" >>/root/.bashrc
export EDITOR='nano'
export LS_OPTIONS='--color=auto'
eval "`dircolors`"
alias ls='ls $LS_OPTIONS'
alias ll='ls $LS_OPTIONS -l'
alias l='ls $LS_OPTIONS -lA'
alias rm='rm -i'
alias cp='cp -i'
alias mv='mv -i'
vmconf='/etc/webmin/miniserv.conf'
alias mysqladmin='mysqladmin --defaults-extra-file=/usr/local/directadmin/conf/my.cnf'
alias mysqldump='mysqldump --defaults-extra-file=/usr/local/directadmin/conf/my.cnf'
alias mysql='mysql --defaults-extra-file=/usr/local/directadmin/conf/my.cnf'
alias lh='history |grep ' 
alias search='find / -name '
alias doserver='curl -o vm_setup_deb.sh -L https://files.delaintech.com/vm_setup_deb.sh && bash vm_setup_deb.sh'
eol
		printf "Print Profile File.\n"
		cat /root/.bashrc
		printf "Done.\n"
	else
		printf "Profile already setup.  See Below.\n"
		cat /root/.bashrc
	fi
}
settimezone() {
	printf "What TimeZone are you in? [America/Chicago]: "
	read -r tmzone
	tmzone=${tmzone:-America/Chicago}
	grep -E "^$tmzone" /etc/timezone >/dev/null
	if [ $? -eq 0 ]; then
		printf "%s found\n" "$tmzone"
	else
		timedatectl set-timezone "$tmzone"
		printf "We set timezone as:\n"
		timedatectl
		printf "Done.\n"
	fi
}
creathostfile() {
	# does the Host already exist?
	grep -q ${hstdomainname} /etc/hosts
	# if not then create it
	if [ $? -ne 0 ]; then
		printf 'Hostfile not found. adding...\n'
		#-----------------------------------------------------------------------------#
		# Set your hostfile info inside the eol																				#
		#----------------------------------------------------------------------------
		cat <<"eol" >>/etc/hosts
209.126.81.64           apollo.delainhosting.com apollo
209.145.52.110          athena.delainhosting.com athena
144.91.108.77           thor.delainhosting.com thor
116.202.102.0           saturn.delainhosting.com saturn
eol
		printf "Print Host File.\n"
		cat /etc/hosts
		printf "Done.\n"
	else
		printf "Hostfile exsits.\nSee Below.\n"
		cat /etc/hosts
	fi
}
#-----------------------------------------------------------------------------#
# Set hostname for Server fdqn.server.net																			#
#-----------------------------------------------------------------------------#
creathostname() {
	# does the Host already exist?
	unset new_hostname
	printf "Please enter a Hostname to add: "
	read -r new_hostname
	grep -q "$new_hostname" /etc/hosts >/dev/null
	if [ $? -ne 0 ]; then
		printf 'Hostname not found. adding...\n'
		printf "Changing hostname %s from to %s...\n" "$cur_hostname" "$new_hostname"
		hostnamectl set-hostname "$new_hostname"
		serverip="$(hostname -I | awk '{print $1}')"
		serverip6="$(hostname -I | awk '{print $2}')"
		servername="$(hostname -s)"
		echo "${serverip}           ${new_hostname} ${servername}" >>/etc/hosts
		echo "${serverip6}          ${new_hostname} ${servername}" >>/etc/hosts
		printf "Print Host File.\n"
		cat /etc/hosts
		printf "\nDone.\n"
	else
		printf "Hostname exsits.\nAll good.\n"
	fi
}
removehosts() {
	printf "Here is the Host file.\n"
	cat /etc/hosts
	printf "\nWhat is the server name or ip to remove? "
	read -r removehosts
	sed -i.bkp '/'$removehosts'/d
' /etc/hosts
	printf "Print Host.\n"
	cat /etc/hosts
	printf "\nDone.\n"
	while true; do
		printf "Continue removing? (y/n)?"
		read -r yn
		yn=${yn:-n}
		case $yn in
		[Yy]*)
			removehosts
			break
			;;
		[Nn]*) break ;;
		esac
	done
}
addhosts() {
	# does the Host already exist?
	unset serverip add_hostname servername
	printf "Enter Hostname to add:[ IP FDQN Hostname ]: "
	read -r serverip add_hostname servername
	grep -q "$add_hostname" /etc/hosts
	# if not then create it
	if [ $? -ne 0 ]; then
		printf 'Hostname not found. adding...\n'
		printf "${serverip}			${add_hostname} ${servername}" | tee -a /etc/hosts
		printf "Print Host File.\n"
		cat /etc/hosts
		printf "Done.\n"
	else
		printf "Hostname exsits.\nAll good.\n"
	fi
	while true; do
		printf "Continue adding? (y/n)?"
		read -r yn
		yn=${yn:-n}
		case $yn in
		[Yy]*)
			creathostentry
			break
			;;
		[Nn]*) break ;;
		esac
	done
}
#-----------------------------------------------------------------------------#
# Create Swap file 								        																		#
#-----------------------------------------------------------------------------#
creatswapfile() {
	printf 'Enter Swapfile size in GB: '
	read -r swapsize
	swapsize=${swapsize:-2}
	printf "You choose %s GB for swap.\n" "$swapsize"
	# does the swap file already exist?
	cp /etc/fstab /etc/fstab.bak
	# if not then create it
	grep -q "swap" /etc/fstab
	if [ $? -ne 0 ]; then
		printf 'Swap file not found.\nCreating Swap file.\n'
		fallocate -l ${swapsize}G /swapfile
		chmod 600 /swapfile
		mkswap /swapfile
		swapon /swapfile
		printf "/swapfile none swap defaults 0 0" | tee -a /etc/fstab
		printf "\nDone\nSwap should be active.\nIf not reboot.\n"
	else
		printf 'Swap file found.\nNo changes made.\n'
	fi
}

# ------------------------------------------------------------------------------#
# Install Functions 															#
# ------------------------------------------------------------------------------#
serverupdate() {
	printf "Debian Cleaning and Updating.\n"
	apt-get ${aptargs} autoremove
	apt-get autoclean
	apt-get ${aptargs} update
	apt-get ${aptargs} upgrade
	apt-get ${aptargs} dist-upgrade
	printf "Done.\n"
	swapoff -a
	sleep 10
	swapon -a
}
installpreq() {
	#DH Added Standard packages
	apt-get install ${aptargs} htop screen inotify-tools ipset pigz pbzip2 tuned rsync zip unzip
	printf "Installing required Virtualmin Config!\n"
}
installvirtualmin() {
	if [ ! -f "$vm" ]; then
		printf "No problem, let's get Virtualmin installed first...\nThis could take a minute...\nor two... or thirty...\nPlease wait ...\n"
		cd $vmsetupbuild || return
		wget http://software.virtualmin.com/gpl/scripts/install.sh
		sh install.sh -v
	else
		printf "Script killed, Virtualmin installed.\n"
		exit
	fi
}
hardenserver() {
	printf "Do we need to Secure sshd? (y/n)? "
	read -r yn
	yn=${yn:-n}
	case $yn in
	[Yy]*)
		cursshport="$(grep -m1 -E "Port .*" /etc/ssh/sshd_config)"
		printf "Enter SSH port to change to: "
		read -r sshport
		sshport=${sshport:-14}
		printf "Set to Port: %s\n" "$sshport"
		printf "Securing the server, please wait...\n"
		sed -i -e "s/$cursshport/Port ${sshport}/g" /etc/ssh/sshd_config
		sed -i -e 's/.*UseDNS .*/UseDNS no/g' /etc/ssh/sshd_config
		sed -i -e 's/#AddressFamily any/AddressFamily inet/g' /etc/ssh/sshd_config
		sed -i -e 's/#LoginGraceTime 2m/LoginGraceTime 2m/g' /etc/ssh/sshd_config
		sed -i -e 's/#MaxAuthTries 6/MaxAuthTries 5/g' /etc/ssh/sshd_config
		sed -i -e 's/#MaxStartups 10:30:100/MaxStartups 10:30:100/g' /etc/ssh/sshd_config
		sed -i -e 's/.*PermitRootLogin yes/PermitRootLogin prohibit-password/g' /etc/ssh/sshd_config
		sed -i -e 's/.*PasswordAuthentication .*/PasswordAuthentication no/g' /etc/ssh/sshd_config
		sed -i -e 's/#ClientAliveInterval .*/ClientAliveInterval 120/g' /etc/ssh/sshd_config
		sed -i -e 's/#ClientAliveCountMax .*/ClientAliveCountMax 15/g' /etc/ssh/sshd_config

		# remove or disable services
		systemctl restart sshd
		;;
	[Nn]*) ;;
	esac
	if [[ $(systemctl is-active rpcbind) = active ]]; then
		printf "rpcbind found.\nRemoving.\n"
		systemctl stop rpcbind
		systemctl disable rpcbind

		printf "Removal complete.\n"
	else
		printf "rpcbind not found.\nNo changes made.\n"
	fi
	if [[ $(systemctl is-active firewalld) = active ]] || [[ $(systemctl is-active fail2ban) = active ]]; then
		printf "Firewalld and or Fail2ban found.\nRemoving.\n"
		systemctl unmask --now firewalld
		systemctl disable firewalld
		systemctl unmask --now fail2ban
		systemctl disable fail2ban
		apt-get ${aptargs} remove firewalld fail2ban
		printf "Removal complete.\n"
	else
		printf "Firewalld and or Fail2ban not found.\nNo changes made.\n"
	fi
	printf "Do we need to intall Maldetect? (y/n)? "
	read -r yn
	yn=${yn:-n}
	case $yn in
	[Yy]*)
		installmaldetect
		;;
	[Nn]*) ;;
	esac
}
installfirewall() {
	printf "Securing the server, please wait...\n"
	cd $vmsetupbuild || exit
	rm -fv csf.tgz
	wget -rnH http://files.delaintech.com/vm-csf-install.sh -P ${vmsetupbuild} >>${logfile}
	cd $vmsetupbuild || exit
	chmod 755 vm-csf-install.sh
	sh vm-csf-install.sh
}
installioncube() {
	# Check if PHP CLI is installed
	hash php 2>/dev/null || {
		echo >&2 "The script requires php-cli but it's not installed.  Aborting."
		exit 1
	}

	# Find the system architecture
	DPKG_ARCH=$(dpkg --print-architecture)
	if [[ "$DPKG_ARCH" = "amd64" ]]; then
		ARCH="x86-64"
	else
		printf "Not supported.\n"
	fi

	# Download and extract
	wget -q -O - "https://downloads.ioncube.com/loader_downloads/ioncube_loaders_lin_${ARCH}.tar.gz" | tar -xzf - -C /usr/local

	# Find PHP version
	PHP_V=$(php -v)
	PHP_VERSION=${PHP_V:4:3}

	# Add the IonCube loader to the PHP configuration
	cat >"/etc/php/${PHP_VERSION}/mods-available/ioncube.ini" <<eol
; configuration for php ioncube module
; priority=10
zend_extension=/usr/local/ioncube/ioncube_loader_lin_$PHP_VERSION.so
eol
	phpenmod -s cli ioncube

	# Restart services
	for i in php${PHP_VERSION}-fpm nginx apache2; do
		service --status-all | grep -Fq "$i" && systemctl restart "$i"
	done
}
installmysqltuner() {
	cd /usr/bin || exit
	wget http://mysqltuner.pl/ -O mysqltuner
	wget https://raw.githubusercontent.com/major/MySQLTuner-perl/master/basic_passwords.txt -O basic_passwords.txt
	wget https://raw.githubusercontent.com/major/MySQLTuner-perl/master/vulnerabilities.csv -O vulnerabilities.csv
	chmod +x mysqltuner
}
installmaldetect() {
	cd $vmsetupbuild || return
	wget https://www.rfxn.com/downloads/maldetect-current.tar.gz
	tar -xzf maldetect-*.tar.gz
	rm -rf maldetect-*.tar.gz
	cd maldetect* || return
	sh install.sh
	#------------------------------------------------------------------------------#
	# Pull in your conf.maldet here. Change the links.
	#------------------------------------------------------------------------------#
	wget https://files.delaintech.com/conf.maldet -O conf.maldet
	\cp -f conf.maldet /usr/local/maldetect/
	maldet -u
}

installinstallatron() {
	cd $vmsetupbuild || exit
	wget https://data.installatron.com/installatron-plugin.sh
	chmod +x installatron-plugin.sh
	./installatron-plugin.sh -f
}
installkernelcare() {
	#Is Linux Kernel Compatible?
	printf "Linux Kernel Compatible?\n"
	comp="$(curl -s -L https://kernelcare.com/checker | python)"
	#What install KernelCare
	if [ "$comp" = "COMPATIBLE" ]; then
		printf "Yes..System Kernel Compatible.\nContinue on....\n"
		curl -s -L https://kernelcare.com/installer | bash
		printf "Kernel Care installed.\n"
		printf 'Enter KernelCare license key: '
		read -r kckey
		kc_key=${kckey:-JOfCWN9tFkOYi1sl}
		/usr/bin/kcarectl --register $kc_key
		/usr/bin/kcarectl --update
	else
		printf "System runs on unsupported kernel. Not installed...\n"
	fi
}
confapache2() {
	printf "Setting up Apache.\n"
	a2enmod http2 headers expires
	printf "Adding KeepAlive config in Apache2.\n"
	sed -i -e 's/Timeout .*/Timeout 60/g' /etc/apache2/apache2.conf
	sed -i -e 's/KeepAliveTimeout .*/KeepAliveTimeout 2/g' /etc/apache2/apache2.conf
	printf "Adding MPM event config in Apache2.\n"
	cat >/etc/apache2/mods-available/mpm_event.conf <<eol
<IfModule mpm_event_module>
    StartServers             6
    MinSpareThreads          32
    MaxSpareThreads          128
    ThreadsPerChild          64
    ServerLimit              32
    MaxRequestWorkers        2048
    MaxConnectionsPerChild   10000
</IfModule>
eol
	printf "Adding mod deflate config in Apache2.\n"
	cat >/etc/apache2/mods-available/deflate.conf <<eol
<IfModule mod_deflate.c>
	<IfModule mod_filter.c>
		AddOutputFilterByType DEFLATE text/html text/plain text/xml text/css
		AddOutputFilterByType DEFLATE application/x-javascript application/javascript application/ecmascript
		AddOutputFilterByType DEFLATE application/rss+xml
		AddOutputFilterByType DEFLATE application/xml
    DeflateCompressionLevel 7
		DeflateMemLevel 8
		DeflateWindowSize 10
	</IfModule>
</IfModule>
eol
	printf "Adding HTST header config in Apache2.\n"
	cat >/etc/apache2/mods-available/headers.conf <<eol
#Set HTST header
Header always set Strict-Transport-Security max-age=31536000
eol
	if [ ! -f "/etc/apache2/mods-enabled/headers.conf" ]; then
		ln -s /etc/apache2/mods-available/headers.conf /etc/apache2/mods-enabled/headers.conf
	else
		printf "HTST header config exsist in Apache2.\n"
	fi
	printf "Adding mod expires config in Apache2.\n"
	cat >/etc/apache2/mods-available/expires.conf <<eol
#set Browser cache aging
<IfModule mod_expires.c>
  ExpiresActive On

  # Images
  ExpiresByType image/jpeg "access plus 1 year"
  ExpiresByType image/gif "access plus 1 year"
  ExpiresByType image/png "access plus 1 year"
  ExpiresByType image/webp "access plus 1 year"
  ExpiresByType image/svg+xml "access plus 1 year"
  ExpiresByType image/x-icon "access plus 1 year"

  # Video
  ExpiresByType video/webm "access plus 1 year"
  ExpiresByType video/mp4 "access plus 1 year"
  ExpiresByType video/mpeg "access plus 1 year"

  # Fonts
  ExpiresByType font/ttf "access plus 1 year"
  ExpiresByType font/otf "access plus 1 year"
  ExpiresByType font/woff "access plus 1 year"
  ExpiresByType font/woff2 "access plus 1 year"
  ExpiresByType application/font-woff "access plus 1 year"

  # CSS, JavaScript
  ExpiresByType text/css "access plus 1 year"
  ExpiresByType text/javascript "access plus 1 year"
  ExpiresByType application/javascript "access plus 1 year"

  # Others
  ExpiresByType application/pdf "access plus 1 year"
  ExpiresByType image/vnd.microsoft.icon "access plus 1 year"
</IfModule>
eol
	if [ ! -f "/etc/apache2/mods-enabled/expires.conf" ]; then
		ln -s /etc/apache2/mods-available/expires.conf /etc/apache2/mods-enabled/expires.conf
	else
		printf "Mod Expires config exsist in Apache2.\n"
	fi
	printf "Adding OCSP in Apache2.\n"
	grep -q "SSLUseStapling on" /etc/apache2/mods-available/ssl.conf
	if [ $? -ne 0 ]; then
		sed -i.bkp '/#SSLStrictSNIVHostCheck On/a\
SSLUseStapling on\
SSLStaplingCache shmcb:/tmp/stapling_cache(128000)\
' /etc/apache2/mods-available/ssl.conf
	else
		printf "OCSP present in Apache2.\n"
	fi
	printf "Do we need to intall IONcube Loaders? (y/n)? "
	read -r yn
	yn=${yn:-n}
	case $yn in
	[Yy]*)
		installioncube
		;;
	[Nn]*) ;;
	esac
	printf "Setting up Apache2.\n"
	systemctl enable apache2
	systemctl restart apache2
	printf "Done.\n"
}
confmail() {
	apt install ${aptargs} pflogsumm opendkim
	printf "Set Root mail? (y/n)? "
	read -r yn
	yn=${yn:-n}
	case $yn in
	[Yy]*)
		printf "Where should root email go: "
		read -r rootemail
		rootemail=${rootemail:-tech@delainhosting.com}
		grep -E "$rootemail" /etc/aliases >/dev/null
		if [ $? -eq 0 ]; then
			printf "%s found already set in Aliases file.\n" "$rootemail"
		else
			echo root: ${rootemail} >>/etc/aliases
			newaliases
			printf "Email set to: "
			grep -E "$rootemail" /etc/aliases
		fi
		;;
	[Nn]*) ;;
	esac

	#	mkdir -p /usr/local/directadmin/custombuild/custom/roundcube/
	#	cp /var/www/html/roundcube/config/config.inc.php ${cbblddir}/custom/roundcube/config.inc.php
	#	sed -i -e 's|"'localhost'"|tls://%n|g' custom/roundcube/config.inc.php
	#	sed -i -e 's/Roundcube Webmail/Delain Hosting Webmail/g' ${cbblddir}/custom/roundcube/config.inc.php
	#
	#	printf "Need identity switch in Roundcube? (yn)"
	#	read -r yn
	#	case $yn in
	#	[Yy]*)
	#		printf "********Is the Firewall on Saturn open?*********\n"
	#		sleep 10
	#		cd ${cbblddir}/custom || exit
	#		cat /usr/local/directadmin/scripts/setup.txt
	#		mysqldump -uroot -p da_roundcube >da_roundcube.bk.sql
	#		wget -rnH http://files.delaintech.com/mysql.initial.sql -P /tmp
	#		mysql -uroot -p da_roundcube </tmp/mysql.initial.sql
	#		wget -rnH --cut-dirs=2 -nc --user='files@delaintech.com' --password='|aaWiS>3wg' ftp://files.delaintech.#com/public_html/files/roundcube
	#
	#		cd ${cbblddir}/custom/roundcube || exit
	#		cp config.inc.php config.inc.php.bk
	#		grep -q "'ident_switch'," config.inc.php
	#		if [ $? -ne 0 ]; then
	#			sed -i.bkp "/'zipdownload',/a\
	#'emoticons',\
	#'identity_select',\
	#'ident_switch',\
	#'markasjunk',\
	#" config.inc.php
	#			${cb} roundcube
	#		fi
	#		;;
	#	[Nn]*) ;;
	#	esac
	#	read -p "Full text Search? (yn)" yn
	#	case $yn in
	#	[Yy]*)
	#		${cb} fts-xapian
	#		${cb} dovecot
	#		;;
	#	[Nn]*) ;;
	#	esac
	cat >/etc/dovecot/conf.d/10-mail.conf <<"eol"
mail_location = maildir:~/Maildir
mail_privileged_group = mail
protocol !indexer-worker {
	#mail_vsize_bg_after_count = 0
}
#Standard NameSpace for Maildir
namespace inbox {
  type = private
  separator = .
  subscriptions = yes
  inbox = yes
}
eol
	cat >/etc/dovecot/conf.d/15-mailboxes.conf <<"eol"
#Standard Folders for Maildir. Namespace in 10-mail.conf
namespace inbox {
  mailbox Drafts {
    auto = subscribe
    special_use = \Drafts
  }
  mailbox Junk {
		auto = subscribe
		special_use = \Junk
}
mailbox Archive {
		auto = subscribe
		special_use = \Archive
}
mailbox Trash {
		auto = subscribe
		special_use = \Trash
}
# For \Sent mailboxes there are two widely used names. We'll mark both of
# them as \Sent and use Sent. User typically deletes one of them if duplicates are created.
mailbox Sent {
		auto = subscribe
		special_use = \Sent
}
mailbox "Sent Messages" {
    special_use = \Sent
  }
}
eol
	printf "Cycling PostFix and Dovecot.\n"
	systemctl restart postfix
	systemctl restart dovecot
}

confnamed() {
	sed -i -e "s|.*listen-on-v6[[:blank:]]*.{ any; };||g" /etc/bind/named.conf.options
	sed -i -e "s|.*allow-transfer[[:blank:]]*.{ none; };||g" /etc/bind/named.conf.options
	grep -q "recursion no;" /etc/bind/named.conf.options
	if [ $? -ne 0 ]; then
		cp /etc/bind/named.conf.options /etc/bind/named.conf.options.bak
		sed -i.bkp '/dnssec-validation auto;/a\
		dnssec-enable yes;\
		listen-on { '$serverip'; };\
		recursion no;\
		notify yes;\
		allow-transfer  { 209.145.52.110; 144.91.108.77; };\
		also-notify     { 209.145.52.110; 144.91.108.77; };\
		version "Not Found";\
		' /etc/bind/named.conf.options
	fi

	apt-get ${aptargs} remove resolvconf
	printf "Setup Resolver configuration.\n"
	cat >/etc/resolv.conf <<eol
search domain $svrdomainname
nameserver 127.0.0.1
nameserver 9.9.9.9
nameserver 149.112.112.112
eol

	cat /etc/resolv.conf
	printf "Restarting Services.\n"
	systemctl enable bind9
	systemctl restart bind9
	printf "Done.\n"
}
confproftpd() {
	printf "Setting up Proftpd.\n"
	sed -i -e "s/Port .*/Port 1414/g" /etc/proftpd/conf.d/virtualmin.conf
	printf "Setting up Proftp.\n"
	systemctl enable proftpd
	systemctl restart proftpd
	printf "Done.\n"
}
confmysql() {
	if ! grep -q 'local-infile=0' /etc/mysql/my.cnf; then
		echo 'local-infile=0' >>/etc/mysql/my.cnf
		systemctl enable mariadb
		systemctl restart mariadb
	fi
	printf "Do we need to intall Mysqltuner? (y/n)? "
	read -r yn
	yn=${yn:-n}
	case $yn in
	[Yy]*)
		installmysqltuner
		;;
	[Nn]*) ;;
	esac
}
serverstatus() {
	printf "Checking Virtualmin."
	systemctl status virtualmin --no-pager
	systemctl status webmin --no-pager
	systemctl status usermin --no-pager
	systemctl status apache2 --no-pager
	systemctl status postfix --no-pager
	systemctl status dovecot --no-pager
	systemctl status proftpd --no-pager
	systemctl status bind9 --no-pager
	systemctl status sshd --no-pager
	printf "done"
}

backupall() {
	printf 'Running Server file backup.\n'
	if [ -d /sys_backup ] && [ -d /admin_backups ]; then
		chmod 755 /admin_backups
		chmod 755 /sys_backup
		chown admin:admin /admin_backups
		chown admin:admin /sys_backup
		cd /
		tar -cvpzf /sys_backup/backup.tar.gz \
			--exclude=/sys_backup/backup.tar.gz \
			--exclude=/admin_backups \
			--exclude=/root/.c* \
			/usr/local/directadmin/conf \
			/usr/local/directadmin/scripts/custom \
			/usr/local/directadmin/custombuild/custom \
			/usr/local/directadmin/data/templates/custom \
			/usr/local/directadmin/data/templates/mx \
			/usr/local/directadmin/data/admin/packages/ \
			/usr/local/directadmin/data/admin/packages.list \
			/usr/local/directadmin/data/users/admin/packages/ \
			/usr/local/directadmin/data/users/admin/packages.list \
			/usr/local/directadmin/data/admin/backup* \
			/sys_backup/mysql_backups \
			/usr/local/etc \
			/etc \
			/root
		printf "Server Backup done.\n"
	else
		mkdir /admin_backups
		mkdir /sys_backup
		chmod 755 /admin_backups
		chmod 755 /sys_backup
		cd /
		tar -cvpzf /sys_backup/backup.tar.gz \
			--exclude=/sys_backup/backup.tar.gz \
			--exclude=/home/admin/admin_backup \
			--exclude=/admin_backups \
			--exclude=/root/.c* \
			--exclude=/sys_backup/backup.tar.gz \
			--exclude=/home/admin/admin_backup \
			--exclude=/admin_backups \
			--exclude=/root/.c* \
			/usr/local/directadmin/conf \
			/usr/local/directadmin/scripts/custom \
			/usr/local/directadmin/custombuild/custom \
			/usr/local/directadmin/data/templates/custom \
			/usr/local/directadmin/data/templates/mx \
			/usr/local/directadmin/data/admin/packages/ \
			/usr/local/directadmin/data/admin/packages.list \
			/usr/local/directadmin/data/users/admin/packages/ \
			/usr/local/directadmin/data/users/admin/packages.list \
			/usr/local/directadmin/data/admin/backup* \
			/sys_backup/mysql_backups \
			/usr/local/etc \
			/etc \
			/root
		printf "Server Backup done.\n"
	fi
}

# ------------------------------------------------------------------------------#
# Main Direct Admin Functions 											    		#
# ------------------------------------------------------------------------------#
clear
vmsetup_banner
linebreak

#Check that user is root.
if [ "$(id -u)" = "0" ]; then
	printf "We are root. Continue on....\n"
else
	printf "This script must be run as root\n"
	exit 1
fi
#What Distro are you on?
printf "Distro are you on??\n" 2>&1
OS_VER=$(sed 's/\..*//' /etc/debian_version)
if [ "$OS_VER" = 9 ]; then
	printf "This appears to be Debian version $OS_VER which is Good!!\n"
elif [ "$OS_VER" = 10 ]; then
	printf "This appears to be Debian version $OS_VER which is Better!!\n"
#elif [ "$OS_VER" = 11 ]; then
#	printf "This appears to be Debian version $OS_VER which is Great!!\n"
else
	[ "$OS_VER" != 10 ]
	printf "System runs on  unsupported Linux. Exiting...\n"
	exit
fi
#Menus Starts here
linebreak
printf "${YELLOW}"
cat <<eot
	Hostname: $cur_hostname Server IP: $serverip
eot
linebreak
printf "${YELLOW}"
cat <<"eot"
			MAIN SERVER SETUP MENU
eot
linebreak
while true; do
	options=("Add Admin User" "Timezone" "Host file" "Hostname" "Remove hosts" "Add hosts" "Swap file" "Update server" "Install Required Packages" "Install Virtualmin" "Harden Server" "Install Firewall" "Install LetsEncrypt" "Install KernelCare" "Install MalDetect" "Install MySQLTuner" "Install Installatron" "Configure Apache" "Configure Named" "Configure Mail" "Configure Proftpd" "Configure MySql" "Check Server Status" "Build All" "Backup Server Files" "Quit")
	printf "Choose an option: \n"
	select opt in "${options[@]}"; do
		case $REPLY in
		1)
			addadminuser
			break
			;;
		2)
			settimezone
			break
			;;
		3)
			creathostfile
			break
			;;
		4)
			creathostname
			break
			;;
		5)
			removehosts
			break
			;;
		6)
			addhosts
			break
			;;
		7)
			creatswapfile
			break
			;;
		8)
			serverupdate
			break
			;;
		9)
			installpreq
			break
			;;
		10)
			installvirtualmin
			break
			;;
		11)
			hardenserver
			break
			;;
		12)
			installfirewall
			break
			;;
		13)
			installletsencrypt
			break
			;;
		14)
			installkernelcare
			break
			;;
		15)
			installmaldetect
			break
			;;
		16)
			installmysqltuner
			break
			;;
		17)
			installinstallatron
			break
			;;
		18)
			confapache2
			break
			;;
		19)
			confnamed
			break
			;;
		20)
			confmail
			break
			;;
		21)
			confproftpd
			break
			;;
		22)
			confmysql
			break
			;;
		23)
			serverstatus
			break
			;;
		24)
			buildalld
			break
			;;
		25)
			backupall
			break
			;;
		26) break 2 ;;
		*) echo "Invalid option. Try again." >&2 ;;
		esac
	done
done

if [ -d "$vmsetupbuild" ]; then
	rm -rf $vmsetupbuild
fi
mkdir $vmsetupbuild
if [ "$run" ]; then
	${run}
	exit
fi

printf "Cleaning up build files, please wait...\n"
cd ~ || exit
rm -rf $vmsetupbuild
